from __future__ import annotations

import re
from typing import List, Tuple
from typing_extensions import assert_type

from xdg.IniFile import IniFile

# The "get" method is quite complex with many overloads. Check that many forms
# are valid.
# The function definition is:
# def get(self, key, group=None, locale=False, type="string", list=False, strict=False):

# Get str
assert_type(IniFile().get("some_key"), str)
assert_type(IniFile().get("some_key", None), str)
assert_type(IniFile().get("some_key", "group"), str)
assert_type(IniFile().get("some_key", "group", False), str)
assert_type(IniFile().get("some_key", "group", True), str)
assert_type(IniFile().get("some_key", "group", True, "string"), str)
assert_type(IniFile().get("some_key", "group", True, "string", False), str)
assert_type(IniFile().get("some_key", "group", True, "string", False, False), str)
assert_type(IniFile().get("some_key", "group", True, "string", False, True), str)
# Keyword parameters
assert_type(IniFile().get("some_key", group=None), str)
assert_type(IniFile().get("some_key", group="group"), str)
assert_type(IniFile().get("some_key", locale=False), str)
assert_type(IniFile().get("some_key", locale=True), str)
assert_type(IniFile().get("some_key", strict=False), str)
assert_type(IniFile().get("some_key", strict=True), str)
assert_type(IniFile().get("some_key", group="group", locale=True, strict=True), str)
# Explicitly set type as string in keyword parameters.
assert_type(IniFile().get("some_key", type="string"), str)
assert_type(IniFile().get("some_key", group=None, type="string"), str)
assert_type(IniFile().get("some_key", group="group", type="string"), str)
assert_type(IniFile().get("some_key", locale=False, type="string"), str)
assert_type(IniFile().get("some_key", locale=True, type="string"), str)
assert_type(IniFile().get("some_key", strict=False, type="string"), str)
assert_type(IniFile().get("some_key", strict=True, type="string"), str)
assert_type(IniFile().get("some_key", group="group", locale=True, strict=True, type="string"), str)
# Explicitly set list.
assert_type(IniFile().get("some_key", list=False), str)
assert_type(IniFile().get("some_key", group=None, list=False), str)
assert_type(IniFile().get("some_key", group="group", list=False), str)
assert_type(IniFile().get("some_key", locale=False, list=False), str)
assert_type(IniFile().get("some_key", locale=True, list=False), str)
assert_type(IniFile().get("some_key", strict=False, list=False), str)
assert_type(IniFile().get("some_key", strict=True, list=False), str)
assert_type(IniFile().get("some_key", group="group", locale=True, strict=True, list=False), str)
# Explicitly set both.
assert_type(IniFile().get("some_key", list=False, type="string"), str)
assert_type(IniFile().get("some_key", group=None, list=False, type="string"), str)
assert_type(IniFile().get("some_key", group="group", list=False, type="string"), str)
assert_type(IniFile().get("some_key", locale=False, list=False, type="string"), str)
assert_type(IniFile().get("some_key", locale=True, list=False, type="string"), str)
assert_type(IniFile().get("some_key", strict=False, list=False, type="string"), str)
assert_type(IniFile().get("some_key", strict=True, list=False, type="string"), str)
assert_type(IniFile().get("some_key", group="group", locale=True, strict=True, list=False, type="string"), str)

# Get List[str]
assert_type(IniFile().get("some_key", "group", True, "string", True), List[str])
assert_type(IniFile().get("some_key", "group", True, "string", True, False), List[str])
assert_type(IniFile().get("some_key", "group", True, "string", True, True), List[str])
# Keyword parameters
assert_type(IniFile().get("some_key", list=True, group=None), List[str])
assert_type(IniFile().get("some_key", list=True, group="group"), List[str])
assert_type(IniFile().get("some_key", list=True, locale=False), List[str])
assert_type(IniFile().get("some_key", list=True, locale=True), List[str])
assert_type(IniFile().get("some_key", list=True, strict=False), List[str])
assert_type(IniFile().get("some_key", list=True, strict=True), List[str])
assert_type(IniFile().get("some_key", list=True, group="group", locale=True, strict=True), List[str])
# Explicitly set list
assert_type(IniFile().get("some_key", list=True), List[str])
assert_type(IniFile().get("some_key", group=None, list=True), List[str])
assert_type(IniFile().get("some_key", group="group", list=True), List[str])
assert_type(IniFile().get("some_key", locale=False, list=True), List[str])
assert_type(IniFile().get("some_key", locale=True, list=True), List[str])
assert_type(IniFile().get("some_key", strict=False, list=True), List[str])
assert_type(IniFile().get("some_key", strict=True, list=True), List[str])
assert_type(IniFile().get("some_key", group="group", locale=True, strict=True, list=True), List[str])
# Explicitly set both
assert_type(IniFile().get("some_key", list=True, type="string"), List[str])
assert_type(IniFile().get("some_key", group=None, list=True, type="string"), List[str])
assert_type(IniFile().get("some_key", group="group", list=True, type="string"), List[str])
assert_type(IniFile().get("some_key", locale=False, list=True, type="string"), List[str])
assert_type(IniFile().get("some_key", locale=True, list=True, type="string"), List[str])
assert_type(IniFile().get("some_key", strict=False, list=True, type="string"), List[str])
assert_type(IniFile().get("some_key", strict=True, list=True, type="string"), List[str])
assert_type(IniFile().get("some_key", group="group", locale=True, strict=True, list=True, type="string"), List[str])

# Get bool
assert_type(IniFile().get("some_key", "group", True, "boolean"), bool)
assert_type(IniFile().get("some_key", "group", True, "boolean", False), bool)
assert_type(IniFile().get("some_key", "group", True, "boolean", False, False), bool)
assert_type(IniFile().get("some_key", "group", True, "boolean", False, True), bool)
# Keyword parameters
assert_type(IniFile().get("some_key", type="boolean"), bool)
assert_type(IniFile().get("some_key", type="boolean", group=None), bool)
assert_type(IniFile().get("some_key", type="boolean", group="group"), bool)
assert_type(IniFile().get("some_key", type="boolean", locale=False), bool)
assert_type(IniFile().get("some_key", type="boolean", locale=True), bool)
assert_type(IniFile().get("some_key", type="boolean", strict=False), bool)
assert_type(IniFile().get("some_key", type="boolean", strict=True), bool)
assert_type(IniFile().get("some_key", type="boolean", group="group", locale=True, strict=True), bool)
# Explicitly set list
assert_type(IniFile().get("some_key", type="boolean", list=False), bool)
assert_type(IniFile().get("some_key", type="boolean", group=None, list=False), bool)
assert_type(IniFile().get("some_key", type="boolean", group="group", list=False), bool)
assert_type(IniFile().get("some_key", type="boolean", locale=False, list=False), bool)
assert_type(IniFile().get("some_key", type="boolean", locale=True, list=False), bool)
assert_type(IniFile().get("some_key", type="boolean", strict=False, list=False), bool)
assert_type(IniFile().get("some_key", type="boolean", strict=True, list=False), bool)
assert_type(IniFile().get("some_key", type="boolean", group="group", locale=True, strict=True, list=False), bool)

# Get List[bool]
assert_type(IniFile().get("some_key", "group", True, "boolean", True), List[bool])
assert_type(IniFile().get("some_key", "group", True, "boolean", True, False), List[bool])
assert_type(IniFile().get("some_key", "group", True, "boolean", True, True), List[bool])
# Keyword parameters
assert_type(IniFile().get("some_key", type="boolean", list=True), List[bool])
assert_type(IniFile().get("some_key", type="boolean", list=True, group=None), List[bool])
assert_type(IniFile().get("some_key", type="boolean", list=True, group="group"), List[bool])
assert_type(IniFile().get("some_key", type="boolean", list=True, locale=False), List[bool])
assert_type(IniFile().get("some_key", type="boolean", list=True, locale=True), List[bool])
assert_type(IniFile().get("some_key", type="boolean", list=True, strict=False), List[bool])
assert_type(IniFile().get("some_key", type="boolean", list=True, strict=True), List[bool])
assert_type(IniFile().get("some_key", type="boolean", list=True, group="group", locale=True, strict=True), List[bool])

# Get int
assert_type(IniFile().get("some_key", "group", True, "integer"), int)
assert_type(IniFile().get("some_key", "group", True, "integer", False), int)
assert_type(IniFile().get("some_key", "group", True, "integer", False, False), int)
assert_type(IniFile().get("some_key", "group", True, "integer", False, True), int)
# Keyword parameters
assert_type(IniFile().get("some_key", type="integer"), int)
assert_type(IniFile().get("some_key", type="integer", group=None), int)
assert_type(IniFile().get("some_key", type="integer", group="group"), int)
assert_type(IniFile().get("some_key", type="integer", locale=False), int)
assert_type(IniFile().get("some_key", type="integer", locale=True), int)
assert_type(IniFile().get("some_key", type="integer", strict=False), int)
assert_type(IniFile().get("some_key", type="integer", strict=True), int)
assert_type(IniFile().get("some_key", type="integer", group="group", locale=True, strict=True), int)
# Explicitly set list.
assert_type(IniFile().get("some_key", type="integer", list=False), int)
assert_type(IniFile().get("some_key", type="integer", group=None, list=False), int)
assert_type(IniFile().get("some_key", type="integer", group="group", list=False), int)
assert_type(IniFile().get("some_key", type="integer", locale=False, list=False), int)
assert_type(IniFile().get("some_key", type="integer", locale=True, list=False), int)
assert_type(IniFile().get("some_key", type="integer", strict=False, list=False), int)
assert_type(IniFile().get("some_key", type="integer", strict=True, list=False), int)
assert_type(IniFile().get("some_key", type="integer", group="group", locale=True, strict=True, list=False), int)

# Get List[int]
assert_type(IniFile().get("some_key", "group", True, "integer", True), List[int])
assert_type(IniFile().get("some_key", "group", True, "integer", True, False), List[int])
assert_type(IniFile().get("some_key", "group", True, "integer", True, True), List[int])
# Keyword parameters
assert_type(IniFile().get("some_key", type="integer", list=True), List[int])
assert_type(IniFile().get("some_key", type="integer", list=True, group=None), List[int])
assert_type(IniFile().get("some_key", type="integer", list=True, group="group"), List[int])
assert_type(IniFile().get("some_key", type="integer", list=True, locale=False), List[int])
assert_type(IniFile().get("some_key", type="integer", list=True, locale=True), List[int])
assert_type(IniFile().get("some_key", type="integer", list=True, strict=False), List[int])
assert_type(IniFile().get("some_key", type="integer", list=True, strict=True), List[int])
assert_type(IniFile().get("some_key", type="integer", list=True, group="group", locale=True, strict=True), List[int])

# Get float
assert_type(IniFile().get("some_key", "group", True, "numeric"), float)
assert_type(IniFile().get("some_key", "group", True, "numeric", False), float)
assert_type(IniFile().get("some_key", "group", True, "numeric", False, False), float)
assert_type(IniFile().get("some_key", "group", True, "numeric", False, True), float)
# Keyword parameters
assert_type(IniFile().get("some_key", type="numeric"), float)
assert_type(IniFile().get("some_key", type="numeric", group=None), float)
assert_type(IniFile().get("some_key", type="numeric", group="group"), float)
assert_type(IniFile().get("some_key", type="numeric", locale=False), float)
assert_type(IniFile().get("some_key", type="numeric", locale=True), float)
assert_type(IniFile().get("some_key", type="numeric", strict=False), float)
assert_type(IniFile().get("some_key", type="numeric", strict=True), float)
assert_type(IniFile().get("some_key", type="numeric", group="group", locale=True, strict=True), float)
# Explicitly set list.
assert_type(IniFile().get("some_key", type="numeric", list=False), float)
assert_type(IniFile().get("some_key", type="numeric", group=None, list=False), float)
assert_type(IniFile().get("some_key", type="numeric", group="group", list=False), float)
assert_type(IniFile().get("some_key", type="numeric", locale=False, list=False), float)
assert_type(IniFile().get("some_key", type="numeric", locale=True, list=False), float)
assert_type(IniFile().get("some_key", type="numeric", strict=False, list=False), float)
assert_type(IniFile().get("some_key", type="numeric", strict=True, list=False), float)
assert_type(IniFile().get("some_key", type="numeric", group="group", locale=True, strict=True, list=False), float)

# Get List[float]
assert_type(IniFile().get("some_key", "group", True, "numeric", True), List[float])
assert_type(IniFile().get("some_key", "group", True, "numeric", True, False), List[float])
assert_type(IniFile().get("some_key", "group", True, "numeric", True, True), List[float])
# Keyword parameters
assert_type(IniFile().get("some_key", type="numeric", list=True), List[float])
assert_type(IniFile().get("some_key", type="numeric", list=True, group=None), List[float])
assert_type(IniFile().get("some_key", type="numeric", list=True, group="group"), List[float])
assert_type(IniFile().get("some_key", type="numeric", list=True, locale=False), List[float])
assert_type(IniFile().get("some_key", type="numeric", list=True, locale=True), List[float])
assert_type(IniFile().get("some_key", type="numeric", list=True, strict=False), List[float])
assert_type(IniFile().get("some_key", type="numeric", list=True, strict=True), List[float])
assert_type(IniFile().get("some_key", type="numeric", list=True, group="group", locale=True, strict=True), List[float])

# Get regex
assert_type(IniFile().get("some_key", "group", True, "regex"), re.Pattern[str])
assert_type(IniFile().get("some_key", "group", True, "regex", False), re.Pattern[str])
assert_type(IniFile().get("some_key", "group", True, "regex", False, False), re.Pattern[str])
assert_type(IniFile().get("some_key", "group", True, "regex", False, True), re.Pattern[str])
# Keyword parameters
assert_type(IniFile().get("some_key", type="regex"), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", group=None), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", group="group"), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", locale=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", locale=True), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", strict=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", strict=True), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", group="group", locale=True, strict=True), re.Pattern[str])
# Explicitly set list.
assert_type(IniFile().get("some_key", type="regex", list=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", group=None, list=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", group="group", list=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", locale=False, list=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", locale=True, list=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", strict=False, list=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", strict=True, list=False), re.Pattern[str])
assert_type(IniFile().get("some_key", type="regex", group="group", locale=True, strict=True, list=False), re.Pattern[str])

# Get List[regex]
assert_type(IniFile().get("some_key", "group", True, "regex", True), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", "group", True, "regex", True, False), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", "group", True, "regex", True, True), List[re.Pattern[str]])
# Keyword parameters
assert_type(IniFile().get("some_key", type="regex", list=True), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", type="regex", list=True, group=None), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", type="regex", list=True, group="group"), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", type="regex", list=True, locale=False), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", type="regex", list=True, locale=True), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", type="regex", list=True, strict=False), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", type="regex", list=True, strict=True), List[re.Pattern[str]])
assert_type(IniFile().get("some_key", type="regex", list=True, group="group", locale=True, strict=True), List[re.Pattern[str]])

# Get point
assert_type(IniFile().get("some_key", "group", True, "point"), Tuple[int, int])
assert_type(IniFile().get("some_key", "group", True, "point", False), Tuple[int, int])
assert_type(IniFile().get("some_key", "group", True, "point", False, False), Tuple[int, int])
assert_type(IniFile().get("some_key", "group", True, "point", False, True), Tuple[int, int])
# Keyword parameters
assert_type(IniFile().get("some_key", type="point"), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", group=None), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", group="group"), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", locale=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", locale=True), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", strict=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", strict=True), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", group="group", locale=True, strict=True), Tuple[int, int])
# Explicitly set list.
assert_type(IniFile().get("some_key", type="point", list=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", group=None, list=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", group="group", list=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", locale=False, list=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", locale=True, list=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", strict=False, list=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", strict=True, list=False), Tuple[int, int])
assert_type(IniFile().get("some_key", type="point", group="group", locale=True, strict=True, list=False), Tuple[int, int])

# Get List[point]
assert_type(IniFile().get("some_key", "group", True, "point", True), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", "group", True, "point", True, False), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", "group", True, "point", True, True), List[Tuple[int, int]])
# Keyword parameters
assert_type(IniFile().get("some_key", type="point", list=True), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", type="point", list=True, group=None), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", type="point", list=True, group="group"), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", type="point", list=True, locale=False), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", type="point", list=True, locale=True), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", type="point", list=True, strict=False), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", type="point", list=True, strict=True), List[Tuple[int, int]])
assert_type(IniFile().get("some_key", type="point", list=True, group="group", locale=True, strict=True), List[Tuple[int, int]])



from typing_extensions import assert_type

from networkx.utils.backends import _dispatchable


@_dispatchable
def some_method(int_p: int, str_p: str) -> float:
    return 0.0


# Wrong param / order
some_method("", 0)  # type: ignore
# backend is kw-only
some_method(0, "", None)  # type: ignore
# No backend means no **backend_kwargs allowed
some_method(0, "", backend_specific_kwarg="")  # type: ignore
some_method(0, "", backend=None, backend_specific_kwarg="")  # type: ignore

# Correct usage
assert_type(some_method(0, ""), float)
# type system doesn't allow this yet (see comment in networkx/utils/backends.pyi)
# assert_type(some_method(0, "", backend=None), float)
assert_type(some_method(0, "", backend="custom backend", backend_specific_kwarg=""), float)



from collections.abc import Callable
from typing import Any

def pdb_on_signal(signalnum: int | None = None) -> None: ...
def pdb_on_exception(limit: int = 100) -> None: ...
def wrap_trace(
    obj, hook: Callable[..., Any] = ..., which: str | None = None, events: str | None = None, label: str | None = None
): ...

__all__ = ["pdb_on_signal", "pdb_on_exception", "wrap_trace"]



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from typing import Any, ClassVar, Final
from typing_extensions import TypeAlias

from docutils import ApplicationError, TransformSpec, nodes
from docutils.languages import LanguageImporter

_TransformTuple: TypeAlias = tuple[str, type[Transform], nodes.Node | None, dict[str, Any]]

__docformat__: Final = "reStructuredText"

class TransformError(ApplicationError): ...

class Transform:
    default_priority: ClassVar[int | None]
    document: nodes.document
    startnode: nodes.Node | None
    language: LanguageImporter
    def __init__(self, document: nodes.document, startnode: nodes.Node | None = None) -> None: ...
    def __getattr__(self, name: str, /) -> Incomplete: ...  # method apply is not implemented

class Transformer(TransformSpec):
    transforms: list[_TransformTuple]
    document: nodes.document
    applied: list[_TransformTuple]
    sorted: bool
    components: Mapping[str, TransformSpec]
    serialno: int
    def __init__(self, document: nodes.document): ...
    def add_transform(self, transform_class: type[Transform], priority: int | None = None, **kwargs) -> None: ...
    def add_transforms(self, transform_list: Iterable[type[Transform]]) -> None: ...
    def add_pending(self, pending: nodes.pending, priority: int | None = None) -> None: ...
    def get_priority_string(self, priority: int) -> str: ...
    def populate_from_components(self, components: Iterable[TransformSpec]) -> None: ...
    def apply_transforms(self) -> None: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_isolate", "isolates", "number_of_isolates"]

@_dispatchable
def is_isolate(G: Graph[_Node], n: _Node) -> bool: ...
@_dispatchable
def isolates(G: Graph[_Node]): ...
@_dispatchable
def number_of_isolates(G: Graph[_Node]) -> int: ...



from _typeshed import Incomplete, IndexableBuffer, SliceableBuffer, Unused
from collections.abc import Callable, Mapping
from typing import Any, Final, TypeVar, overload
from typing_extensions import TypeAlias

from pyasn1.codec.ber.encoder import AbstractItemEncoder
from pyasn1.type.tag import TagSet

# Use _typeshed._SupportsGetItemBuffer after PEP 688
_SupportsGetItemBuffer: TypeAlias = SliceableBuffer | IndexableBuffer
_R = TypeVar("_R")
_B = TypeVar("_B", bound=_SupportsGetItemBuffer)
# The possible return type is a union of all other decode methods, ie: AnyOf[Incomplete | bool]
_AllDecodersReturnType: TypeAlias = Any

CLASSES: Final[dict[tuple[bool, bool], int]]

class LDAPBooleanEncoder(AbstractItemEncoder):
    supportIndefLenMode: bool
    # Requires pyasn1 > 0.3.7
    def encodeValue(self, value: bool | int, asn1Spec: Unused, encodeFun: Unused, **options: Unused): ...

customTagMap: dict[TagSet, AbstractItemEncoder]
customTypeMap: dict[int, AbstractItemEncoder]

def compute_ber_size(data): ...
def decode_message_fast(message): ...
@overload
def decode_sequence(message: _B, start: int, stop: int, context_decoders: Mapping[int, Callable[[_B, int, int], _R]]) -> _R: ...
@overload
def decode_sequence(
    message: _SupportsGetItemBuffer, start: int, stop: int, context_decoders: None = None
) -> _AllDecodersReturnType: ...
def decode_integer(message, start: int, stop: int, context_decoders: Unused = None): ...
def decode_octet_string(message, start: int, stop: int, context_decoders: Unused = None): ...
def decode_boolean(message, start: int, stop: int, context_decoders: Unused = None): ...
def decode_bind_response(message, start: int, stop: int, context_decoders: Unused = None): ...
def decode_extended_response(message, start: int, stop: int, context_decoders: Unused = None): ...
def decode_intermediate_response(message, start: int, stop: int, context_decoders: Unused = None): ...
def decode_controls(message, start: int, stop: int, context_decoders: Unused = None): ...
def ldap_result_to_dict_fast(response): ...
def get_byte(x): ...
def get_bytes(x): ...

# The possible return type is a union of all other decode methods, ie: AnyOf[Incomplete | bool]
DECODERS: dict[tuple[int, int], Callable[..., _AllDecodersReturnType]]
BIND_RESPONSE_CONTEXT: dict[int, Callable[..., Incomplete]]
EXTENDED_RESPONSE_CONTEXT: dict[int, Callable[..., Incomplete]]
INTERMEDIATE_RESPONSE_CONTEXT: dict[int, Callable[..., Incomplete]]
LDAP_MESSAGE_CONTEXT: dict[int, Callable[..., Incomplete]]
CONTROLS_CONTEXT: dict[int, Callable[..., Incomplete]]



from types import ModuleType
from typing import Any

def get_original(mod_name: str, item_name: str) -> Any: ...
def patch_item(module: ModuleType, attr: str, newitem: object) -> None: ...
def remove_item(module: ModuleType, attr: str) -> None: ...
def patch_module(target_module: ModuleType, source_module: ModuleType, items: list[str] | None = None) -> bool: ...



from typing import ClassVar

from .core import UnitedStates

class TexasBase(UnitedStates):
    texas_include_confederate_heroes: ClassVar[bool]
    texas_include_independance_day: ClassVar[bool]
    texas_san_jacinto_day: ClassVar[bool]
    texas_emancipation_day: ClassVar[bool]
    texas_lyndon_johnson_day: ClassVar[bool]

class Texas(TexasBase): ...



import datetime
from _typeshed import Unused
from collections.abc import Mapping
from typing import ClassVar, type_check_only

from .exceptions import (
    AmbiguousTimeError as AmbiguousTimeError,
    InvalidTimeError as InvalidTimeError,
    NonExistentTimeError as NonExistentTimeError,
    UnknownTimeZoneError as UnknownTimeZoneError,
)
from .tzinfo import BaseTzInfo as BaseTzInfo, DstTzInfo, StaticTzInfo

# Actually named UTC and then masked with a singleton with the same name
@type_check_only
class _UTCclass(BaseTzInfo):
    def localize(self, dt: datetime.datetime, is_dst: bool | None = False) -> datetime.datetime: ...
    def normalize(self, dt: datetime.datetime, is_dst: bool | None = False) -> datetime.datetime: ...
    def tzname(self, dt: datetime.datetime | None) -> str: ...
    def utcoffset(self, dt: datetime.datetime | None) -> datetime.timedelta: ...
    def dst(self, dt: datetime.datetime | None) -> datetime.timedelta: ...

utc: _UTCclass
UTC: _UTCclass

def timezone(zone: str) -> _UTCclass | StaticTzInfo | DstTzInfo: ...

class _FixedOffset(datetime.tzinfo):
    zone: ClassVar[None]
    def __init__(self, minutes: int) -> None: ...
    def utcoffset(self, dt: Unused) -> datetime.timedelta | None: ...
    def dst(self, dt: Unused) -> datetime.timedelta: ...
    def tzname(self, dt: Unused) -> None: ...
    def localize(self, dt: datetime.datetime, is_dst: bool | None = False) -> datetime.datetime: ...
    def normalize(self, dt: datetime.datetime, is_dst: bool | None = False) -> datetime.datetime: ...

def FixedOffset(offset: int, _tzinfos: dict[int, _FixedOffset] = {}) -> _UTCclass | _FixedOffset: ...

all_timezones: list[str]
all_timezones_set: set[str]
common_timezones: list[str]
common_timezones_set: set[str]
country_timezones: Mapping[str, list[str]]
country_names: Mapping[str, str]
ZERO: datetime.timedelta
HOUR: datetime.timedelta
VERSION: str

__all__ = [
    "timezone",
    "utc",
    "country_timezones",
    "country_names",
    "AmbiguousTimeError",
    "InvalidTimeError",
    "NonExistentTimeError",
    "UnknownTimeZoneError",
    "all_timezones",
    "all_timezones_set",
    "common_timezones",
    "common_timezones_set",
    "BaseTzInfo",
    "FixedOffset",
]



from .jws import JsonWebSignature as JsonWebSignature
from .models import JWSAlgorithm as JWSAlgorithm, JWSHeader as JWSHeader, JWSObject as JWSObject

__all__ = ["JsonWebSignature", "JWSAlgorithm", "JWSHeader", "JWSObject"]



from . import decoder as decoder, encoder as encoder
from .decoder import (
    TomlDecodeError as TomlDecodeError,
    TomlDecoder as TomlDecoder,
    TomlPreserveCommentDecoder as TomlPreserveCommentDecoder,
    load as load,
    loads as loads,
)
from .encoder import (
    TomlArraySeparatorEncoder as TomlArraySeparatorEncoder,
    TomlEncoder as TomlEncoder,
    TomlNumpyEncoder as TomlNumpyEncoder,
    TomlPathlibEncoder as TomlPathlibEncoder,
    TomlPreserveCommentEncoder as TomlPreserveCommentEncoder,
    TomlPreserveInlineDictEncoder as TomlPreserveInlineDictEncoder,
    dump as dump,
    dumps as dumps,
)



from _typeshed import Unused
from collections.abc import Callable, Iterable, Mapping, Sequence
from typing import Any, NamedTuple
from typing_extensions import TypeAlias

from django import forms
from django.db.models import Choices
from django.forms import Widget
from django_stubs_ext import StrOrPromise

DJANGO_50: bool

# Ref: django-stubs/forms/fields.pyi
# Problem: attribute `widget` is always of type `Widget` after field instantiation.
# However, on class level it can be set to `Type[Widget]` too.
# If we annotate it as `Union[Widget, Type[Widget]]`, every code that uses field
# instances will not typecheck.
# If we annotate it as `Widget`, any widget subclasses that do e.g.
# `widget = Select` will not typecheck.
# `Any` gives too much freedom, but does not create false positives.
_ClassLevelWidget: TypeAlias = Any
# Validator parameter type depends on type of the form field used.
_ValidatorCallable: TypeAlias = Callable[[Any], None]
# Based on django-stubs utils/choices.pyi
_Choice: TypeAlias = tuple[Any, Any]
_ChoiceNamedGroup: TypeAlias = tuple[str, Iterable[_Choice]]
_Choices: TypeAlias = Iterable[_Choice | _ChoiceNamedGroup]
_ChoicesMapping: TypeAlias = Mapping[Any, Any]
_ChoicesInput: TypeAlias = _Choices | _ChoicesMapping | type[Choices] | Callable[[], _Choices | _ChoicesMapping]

class RangeField(forms.MultiValueField):
    widget: _ClassLevelWidget = ...
    def __init__(
        self,
        fields: tuple[forms.Field, forms.Field] | None = None,
        *,
        # Inherited from Django MultiValueField
        require_all_fields: bool = True,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: StrOrPromise | None = ...,
        initial: Any | None = ...,  # Type depends on the form field used.
        help_text: StrOrPromise = ...,
        error_messages: Mapping[str, StrOrPromise] | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...  # Args/kwargs can be any field params, passes to parent
    def compress(self, data_list: list[Any] | None) -> slice | None: ...  # Data list elements can be any field value type

class DateRangeField(RangeField):
    widget: _ClassLevelWidget = ...
    def compress(self, data_list: list[Any] | None) -> slice | None: ...  # Date values in list can be any date type

class DateTimeRangeField(RangeField):
    widget: _ClassLevelWidget = ...

class IsoDateTimeRangeField(RangeField):
    widget: _ClassLevelWidget = ...

class TimeRangeField(RangeField):
    widget: _ClassLevelWidget = ...

class Lookup(NamedTuple):
    value: Any  # Lookup values can be any filterable type
    lookup_expr: str

class LookupChoiceField(forms.MultiValueField):
    def __init__(
        self,
        field: forms.Field,
        lookup_choices: Sequence[tuple[str, str]],
        *,
        empty_label: StrOrPromise = ...,
        widget: Unused = ...,
        help_text: Unused = ...,
        # Inherited from Django MultiValueField
        require_all_fields: bool = True,
        required: bool = ...,
        label: StrOrPromise | None = ...,
        initial: Any | None = ...,  # Type depends on the form field used.
        error_messages: Mapping[str, StrOrPromise] | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...  # Args/kwargs can be any field params, uses kwargs for empty_label
    def compress(self, data_list: list[Any] | None) -> Lookup | None: ...  # Data list can contain any lookup components

class IsoDateTimeField(forms.DateTimeField):
    ISO_8601: str
    input_formats: list[str]
    def strptime(self, value: str, format: str) -> Any: ...  # Returns datetime objects or parsing results

class BaseCSVField(forms.Field):
    base_widget_class: _ClassLevelWidget = ...
    def clean(self, value: Any) -> Any: ...  # Cleaned values can be any valid field type

class BaseRangeField(BaseCSVField):
    widget: _ClassLevelWidget = ...
    def clean(self, value: Any) -> Any: ...  # Input and output values can be any range type

class ChoiceIterator:
    field: ChoiceField
    choices: Sequence[tuple[Any, str]]  # Choice values can be any type (int, str, Model, etc.)
    def __init__(
        self, field: ChoiceField, choices: Sequence[tuple[Any, str]]
    ) -> None: ...  # Choice values can be any selectable type
    def __iter__(self) -> Any: ...  # Iterator yields choice tuples with any value types
    def __len__(self) -> int: ...

class ModelChoiceIterator(forms.models.ModelChoiceIterator):
    def __iter__(self) -> Any: ...  # Iterator yields choice tuples with any value types
    def __len__(self) -> int: ...

class ChoiceIteratorMixin:
    null_label: StrOrPromise | None
    null_value: Any  # Null choice values can be any type (None, empty string, etc.)
    def __init__(self, *, null_label: StrOrPromise | None, null_value: Any) -> None: ...

class ChoiceField(ChoiceIteratorMixin, forms.ChoiceField):
    iterator = ChoiceIterator
    empty_label: StrOrPromise
    def __init__(
        self,
        *,
        empty_label: StrOrPromise = ...,
        # Inherited from Django ChoiceField
        choices: _ChoicesInput = (),
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: StrOrPromise | None = ...,
        initial: Any | None = ...,  # Type depends on the form field used.
        help_text: StrOrPromise = ...,
        error_messages: Mapping[str, StrOrPromise] | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
        null_label: StrOrPromise | None,
        null_value: Any,  # Type depends on the form field used.
    ) -> None: ...

class MultipleChoiceField(ChoiceIteratorMixin, forms.MultipleChoiceField):
    iterator = ChoiceIterator
    empty_label: StrOrPromise | None

class ModelChoiceField(ChoiceIteratorMixin, forms.ModelChoiceField[Any]):
    iterator = ModelChoiceIterator
    def to_python(self, value: Any) -> Any: ...  # Converts any input to Python model objects or values

class ModelMultipleChoiceField(ChoiceIteratorMixin, forms.ModelMultipleChoiceField[Any]):
    iterator = ModelChoiceIterator



from typing import Generic, Literal

from ._typing import GeoT
from .lib import Geometry

class PreparedGeometry(Generic[GeoT]):
    context: GeoT
    prepared: Literal[True]
    def __init__(self, context: GeoT | PreparedGeometry[GeoT]) -> None: ...
    def contains(self, other: Geometry | None) -> bool: ...
    def contains_properly(self, other: Geometry | None) -> bool: ...
    def covers(self, other: Geometry | None) -> bool: ...
    def crosses(self, other: Geometry | None) -> bool: ...
    def disjoint(self, other: Geometry | None) -> bool: ...
    def intersects(self, other: Geometry | None) -> bool: ...
    def overlaps(self, other: Geometry | None) -> bool: ...
    def touches(self, other: Geometry | None) -> bool: ...
    def within(self, other: Geometry | None) -> bool: ...

def prep(ob: GeoT | PreparedGeometry[GeoT]) -> PreparedGeometry[GeoT]: ...



from collections.abc import Generator
from typing import overload

from networkx.algorithms.shortest_paths.weighted import _WeightFunc
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "shortest_path",
    "all_shortest_paths",
    "single_source_all_shortest_paths",
    "all_pairs_all_shortest_paths",
    "shortest_path_length",
    "average_shortest_path_length",
    "has_path",
]

@_dispatchable
def has_path(G: Graph[_Node], source: _Node, target: _Node) -> bool: ...
@overload  # both source and target are specified => (s -> t)
def shortest_path(
    G: Graph[_Node],
    source: _Node,
    target: _Node,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> list[_Node]: ...
@overload  # only source is specified => {t1: (s -> t), t2: (s -> t), ...}
def shortest_path(
    G: Graph[_Node],
    source: _Node,
    target: None = None,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> dict[_Node, list[_Node]]: ...
@overload  # only target is specified (positional) => {s1: (s1 -> t), s2: (s2 -> t), ...}
def shortest_path(
    G: Graph[_Node],
    source: None,
    target: _Node,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> dict[_Node, list[_Node]]: ...
@overload  # only target is specified (keyword) => {s1: (s1 -> t), s2: (s2 -> t), ...}
def shortest_path(
    G: Graph[_Node],
    source: None = None,
    *,
    target: _Node,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    backend: str | None = None,
    **backend_kwargs,
) -> dict[_Node, list[_Node]]: ...
@overload
def shortest_path(  # source and target are not specified => generator of (t, {s1: (s1 -> t), s2: (s2 -> t), ...})
    G: Graph[_Node],
    source: None = None,
    target: None = None,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> Generator[tuple[_Node, dict[str, list[_Node]]]]: ...
@overload  # both source and target are specified => len(s -> t)
def shortest_path_length(
    G: Graph[_Node],
    source: _Node,
    target: _Node,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> float: ...
@overload  # only source is specified => {t1: len(s -> t1), t2: len(s -> t2), ...}
def shortest_path_length(
    G: Graph[_Node],
    source: _Node,
    target: None = None,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> dict[_Node, float]: ...
@overload  # only target is specified (positional) => {s1: len(s1 -> t), s2: len(s2 -> t), ...}
def shortest_path_length(
    G: Graph[_Node],
    source: None,
    target: _Node,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> dict[_Node, float]: ...
@overload  # only target is specified (keyword) => {s1: len(s1 -> t), s2: len(s2 -> t), ...}
def shortest_path_length(
    G: Graph[_Node],
    source: None = None,
    *,
    target: _Node,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    backend: str | None = None,
    **backend_kwargs,
) -> dict[_Node, float]: ...
@overload
def shortest_path_length(  # source and target are not specified => generator of (t, {s1: len(s1 -> t), s2: len(s2 -> t), ...})
    G: Graph[_Node],
    source: None = None,
    target: None = None,
    weight: str | _WeightFunc[_Node] | None = None,
    method: str | None = "dijkstra",
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> Generator[tuple[_Node, dict[_Node, float]]]: ...
@_dispatchable
def average_shortest_path_length(
    G: Graph[_Node], weight: str | _WeightFunc[_Node] | None = None, method: str | None = None
) -> float: ...
@_dispatchable
def all_shortest_paths(
    G: Graph[_Node], source: _Node, target: _Node, weight: str | _WeightFunc[_Node] | None = None, method: str | None = "dijkstra"
) -> Generator[list[_Node]]: ...
@_dispatchable
def single_source_all_shortest_paths(
    G: Graph[_Node], source: _Node, weight: str | _WeightFunc[_Node] | None = None, method: str | None = "dijkstra"
) -> Generator[tuple[_Node, list[list[_Node]]]]: ...
@_dispatchable
def all_pairs_all_shortest_paths(
    G: Graph[_Node], weight: str | _WeightFunc[_Node] | None = None, method: str | None = "dijkstra"
) -> Generator[tuple[_Node, dict[_Node, list[list[_Node]]]]]: ...



from __future__ import annotations

from typing import Any, KeysView, TypeVar

KT = TypeVar("KT")


class MyKeysView(KeysView[KT]):
    pass


d: dict[Any, Any] = {}
dict_keys = type(d.keys())

# This should not cause an error like `Member "register" is unknown`:
MyKeysView.register(dict_keys)



def registerWidget(widget) -> None: ...
def getCodes(): ...
def getCodeNames(): ...
def createBarcodeDrawing(codeName, **options): ...
def createBarcodeImageInMemory(codeName, **options): ...

__all__ = ("registerWidget", "getCodes", "getCodeNames", "createBarcodeDrawing", "createBarcodeImageInMemory")



from collections.abc import Callable
from typing import TypeVar

from .parser import Parameters

class IcalendarProperty:
    params: Parameters

_T = TypeVar("_T")

def string_parameter(
    name: str,
    doc: str,
    default: Callable[..., str | None] = ...,
    convert: Callable[[str], _T] | None = None,
    convert_to: Callable[[_T], str] | None = None,
) -> property: ...

ALTREP: property
CN: property
CUTYPE: property

def quoted_list_parameter(name: str, doc: str) -> property: ...

DELEGATED_FROM: property
DELEGATED_TO: property
DIR: property
FBTYPE: property
LANGUAGE: property
MEMBER: property
PARTSTAT: property
RANGE: property
RELATED: property
ROLE: property

def boolean_parameter(name: str, default: bool, doc: str) -> property: ...

RSVP: property
SENT_BY: property
TZID: property
RELTYPE: property

__all__ = [
    "string_parameter",
    "quoted_list_parameter",
    "ALTREP",
    "CN",
    "CUTYPE",
    "DELEGATED_FROM",
    "DELEGATED_TO",
    "DIR",
    "FBTYPE",
    "LANGUAGE",
    "MEMBER",
    "PARTSTAT",
    "RANGE",
    "RELATED",
    "ROLE",
    "RSVP",
    "SENT_BY",
    "TZID",
]



from _typeshed import Incomplete, Unused
from collections.abc import Generator, Iterable, Mapping, MutableSet, Reversible
from typing import NoReturn

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _EdgePlus, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["check_planarity", "is_planar", "PlanarEmbedding"]

@_dispatchable
def is_planar(G: Graph[_Node]) -> bool: ...
@_dispatchable
def check_planarity(G: Graph[_Node], counterexample: bool = False): ...
@_dispatchable
def get_counterexample(G: Graph[_Node]) -> Graph[_Node]: ...
@_dispatchable
def get_counterexample_recursive(G: Graph[_Node]) -> Graph[_Node]: ...

class Interval:
    low: Incomplete
    high: Incomplete

    def __init__(self, low=None, high=None) -> None: ...
    def empty(self): ...
    def copy(self): ...
    def conflicting(self, b, planarity_state): ...

class ConflictPair:
    left: Incomplete
    right: Incomplete

    def __init__(self, left=..., right=...) -> None: ...
    def swap(self) -> None: ...
    def lowest(self, planarity_state): ...

class LRPlanarity:
    __slots__ = [
        "G",
        "roots",
        "height",
        "lowpt",
        "lowpt2",
        "nesting_depth",
        "parent_edge",
        "DG",
        "adjs",
        "ordered_adjs",
        "ref",
        "side",
        "S",
        "stack_bottom",
        "lowpt_edge",
        "left_ref",
        "right_ref",
        "embedding",
    ]
    G: Incomplete
    roots: Incomplete
    height: Incomplete
    lowpt: Incomplete
    lowpt2: Incomplete
    nesting_depth: Incomplete
    parent_edge: Incomplete
    DG: Incomplete
    adjs: Incomplete
    ordered_adjs: Incomplete
    ref: Incomplete
    side: Incomplete
    S: Incomplete
    stack_bottom: Incomplete
    lowpt_edge: Incomplete
    left_ref: Incomplete
    right_ref: Incomplete
    embedding: Incomplete

    def __init__(self, G: Graph[_Node]) -> None: ...
    def lr_planarity(self) -> PlanarEmbedding[Incomplete] | None: ...
    def lr_planarity_recursive(self): ...
    def dfs_orientation(self, v): ...
    def dfs_orientation_recursive(self, v) -> None: ...
    def dfs_testing(self, v): ...
    def dfs_testing_recursive(self, v): ...
    def add_constraints(self, ei, e): ...
    def remove_back_edges(self, e) -> None: ...
    def dfs_embedding(self, v): ...
    def dfs_embedding_recursive(self, v) -> None: ...
    def sign(self, e): ...
    def sign_recursive(self, e): ...

# NOTE: Graph subclasses relationships are so complex
# we're only overriding methods that differ in signature from the base classes
# to use inheritance to our advantage and reduce complexity
class PlanarEmbedding(DiGraph[_Node]):
    def get_data(self) -> dict[_Node, list[_Node]]: ...
    def set_data(self, data: Mapping[_Node, Reversible[_Node]]) -> None: ...
    def neighbors_cw_order(self, v: _Node) -> Generator[_Node, None, None]: ...
    def add_half_edge(self, start_node: _Node, end_node: _Node, *, cw: _Node | None = None, ccw: _Node | None = None): ...
    def check_structure(self) -> None: ...
    def add_half_edge_ccw(self, start_node: _Node, end_node: _Node, reference_neighbor: _Node) -> None: ...
    def add_half_edge_cw(self, start_node: _Node, end_node: _Node, reference_neighbor: _Node) -> None: ...
    def connect_components(self, v: _Node, w: _Node) -> None: ...
    def add_half_edge_first(self, start_node: _Node, end_node: _Node) -> None: ...
    def next_face_half_edge(self, v: _Node, w: _Node) -> tuple[_Node, _Node]: ...
    def traverse_face(
        self, v: _Node, w: _Node, mark_half_edges: MutableSet[tuple[_Node, _Node]] | None = None
    ) -> list[_Node]: ...
    # Overriden in __init__ to always raise
    def add_edge(self, u_of_edge: _Node, v_of_edge: _Node, **attr: Unused) -> NoReturn: ...
    def add_edges_from(self, ebunch_to_add: Iterable[_EdgePlus[_Node]], **attr: Unused) -> NoReturn: ...
    def add_weighted_edges_from(
        self, ebunch_to_add: Iterable[tuple[_Node, _Node, float]], weight: str = "weight", **attr: Unused
    ) -> NoReturn: ...



from _typeshed import Incomplete
from collections.abc import Collection

from networkx.utils.backends import _dispatchable

__all__ = [
    "caveman_graph",
    "connected_caveman_graph",
    "relaxed_caveman_graph",
    "random_partition_graph",
    "planted_partition_graph",
    "gaussian_random_partition_graph",
    "ring_of_cliques",
    "windmill_graph",
    "stochastic_block_model",
    "LFR_benchmark_graph",
]

@_dispatchable
def caveman_graph(l, k): ...
@_dispatchable
def connected_caveman_graph(l, k): ...
@_dispatchable
def relaxed_caveman_graph(l, k, p, seed=None): ...
@_dispatchable
def random_partition_graph(sizes, p_in, p_out, seed=None, directed: bool = False): ...
@_dispatchable
def planted_partition_graph(l, k, p_in, p_out, seed=None, directed: bool = False): ...
@_dispatchable
def gaussian_random_partition_graph(n, s, v, p_in, p_out, directed: bool = False, seed=None): ...
@_dispatchable
def ring_of_cliques(num_cliques, clique_size): ...
@_dispatchable
def windmill_graph(n, k): ...
@_dispatchable
def stochastic_block_model(
    sizes,
    p,
    nodelist: Collection[Incomplete] | None = None,
    seed=None,
    directed: bool = False,
    selfloops: bool = False,
    sparse: bool = True,
): ...
@_dispatchable
def LFR_benchmark_graph(
    n,
    tau1,
    tau2,
    mu,
    average_degree=None,
    min_degree=None,
    max_degree=None,
    min_community=None,
    max_community=None,
    tol: float = 1e-07,
    max_iters: int = 500,
    seed=None,
): ...



from collections import OrderedDict
from typing import Any

import click

def index() -> str: ...
def _click_to_tree(
    ctx: click.Context, node: click.Command, ancestors: list[click.Command] | None = None
) -> OrderedDict[str, Any]: ...



from typing import Any

class _PasslibRegistryProxy:
    __name__: str
    __package__: str | None
    def __getattr__(self, attr: str) -> Any: ...  # returns handler that is a value from object.__dict__
    def __setattr__(self, attr: str, value) -> None: ...
    def __dir__(self) -> list[str]: ...

def register_crypt_handler_path(name: str, path: str) -> None: ...
def register_crypt_handler(
    handler: Any, force: bool = False, _attr: str | None = None
) -> None: ...  # expected handler is object with attr handler.name
def get_crypt_handler(name: str, default: Any = ...) -> Any: ...  # returns handler or default
def list_crypt_handlers(loaded_only: bool = False) -> list[str]: ...

__all__ = ["register_crypt_handler_path", "register_crypt_handler", "get_crypt_handler", "list_crypt_handlers"]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.full_type_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CppShapeInferenceResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class HandleShapeAndType(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SHAPE_FIELD_NUMBER: builtins.int
        DTYPE_FIELD_NUMBER: builtins.int
        TYPE_FIELD_NUMBER: builtins.int
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
        @property
        def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
        @property
        def type(self) -> tensorflow.core.framework.full_type_pb2.FullTypeDef: ...
        def __init__(
            self,
            *,
            shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
            dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
            type: tensorflow.core.framework.full_type_pb2.FullTypeDef | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["shape", b"shape", "type", b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["dtype", b"dtype", "shape", b"shape", "type", b"type"]) -> None: ...

    @typing.final
    class HandleData(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        IS_SET_FIELD_NUMBER: builtins.int
        SHAPE_AND_TYPE_FIELD_NUMBER: builtins.int
        is_set: builtins.bool
        @property
        def shape_and_type(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            global___CppShapeInferenceResult.HandleShapeAndType
        ]:
            """Only valid if <is_set>."""

        def __init__(
            self,
            *,
            is_set: builtins.bool | None = ...,
            shape_and_type: collections.abc.Iterable[global___CppShapeInferenceResult.HandleShapeAndType] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["is_set", b"is_set", "shape_and_type", b"shape_and_type"]) -> None: ...

    SHAPE_FIELD_NUMBER: builtins.int
    HANDLE_DATA_FIELD_NUMBER: builtins.int
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    @property
    def handle_data(self) -> global___CppShapeInferenceResult.HandleData: ...
    def __init__(
        self,
        *,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        handle_data: global___CppShapeInferenceResult.HandleData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["handle_data", b"handle_data", "shape", b"shape"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["handle_data", b"handle_data", "shape", b"shape"]) -> None: ...

global___CppShapeInferenceResult = CppShapeInferenceResult

@typing.final
class CppShapeInferenceInputsNeeded(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INPUT_TENSORS_NEEDED_FIELD_NUMBER: builtins.int
    INPUT_TENSORS_AS_SHAPES_NEEDED_FIELD_NUMBER: builtins.int
    @property
    def input_tensors_needed(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def input_tensors_as_shapes_needed(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        input_tensors_needed: collections.abc.Iterable[builtins.int] | None = ...,
        input_tensors_as_shapes_needed: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "input_tensors_as_shapes_needed", b"input_tensors_as_shapes_needed", "input_tensors_needed", b"input_tensors_needed"
        ],
    ) -> None: ...

global___CppShapeInferenceInputsNeeded = CppShapeInferenceInputsNeeded



from _typeshed import Incomplete

class UnsupportedData(TypeError): ...

class InputType:
    def __init__(self, *a, **kw) -> None: ...
    def get_entry_seq(self, data_seq, headers): ...

class DictInputType(InputType):
    def check_type(self, obj): ...
    def guess_headers(self, obj): ...
    def get_entry(self, obj, headers): ...
    def get_entry_seq(self, obj, headers): ...

class ObjectInputType(InputType):
    def check_type(self, obj): ...
    def guess_headers(self, obj): ...
    def get_entry(self, obj, headers): ...

class ListInputType(InputType):
    def check_type(self, obj): ...
    def guess_headers(self, obj) -> None: ...
    def get_entry(self, obj, headers): ...
    def get_entry_seq(self, obj_seq, headers): ...

class TupleInputType(InputType):
    def check_type(self, obj): ...
    def guess_headers(self, obj) -> None: ...
    def get_entry(self, obj, headers): ...
    def get_entry_seq(self, obj_seq, headers): ...

class NamedTupleInputType(InputType):
    def check_type(self, obj): ...
    def guess_headers(self, obj): ...
    def get_entry(self, obj, headers): ...
    def get_entry_seq(self, obj_seq, headers): ...

class Table:
    headers: Incomplete
    metadata: Incomplete
    def __init__(self, data=None, headers=..., metadata=None) -> None: ...
    def extend(self, data) -> None: ...
    @classmethod
    def from_dict(cls, data, headers=..., max_depth: int = 1, metadata=None): ...
    @classmethod
    def from_list(cls, data, headers=..., max_depth: int = 1, metadata=None): ...
    @classmethod
    def from_object(cls, data, headers=..., max_depth: int = 1, metadata=None): ...
    @classmethod
    def from_data(cls, data, headers=..., max_depth: int = 1, **kwargs): ...
    def __len__(self): ...
    def __getitem__(self, idx): ...
    def to_html(
        self,
        orientation=None,
        wrapped: bool = True,
        with_headers: bool = True,
        with_newlines: bool = True,
        with_metadata: bool = False,
        max_depth: int = 1,
    ): ...
    def get_cell_html(self, value): ...
    def to_text(self, with_headers: bool = True, maxlen=None): ...

__all__ = ["Table"]



from _typeshed import Incomplete
from collections.abc import Generator

from reportlab.pdfbase.pdfdoc import PDFObject

class PDFPattern(PDFObject):
    __RefOnly__: int
    pattern: Incomplete
    arguments: Incomplete
    def __init__(self, pattern_sequence, **keywordargs) -> None: ...
    def __setitem__(self, item, value) -> None: ...
    def __getitem__(self, item): ...
    def eval(self, L) -> Generator[Incomplete, None, None]: ...
    def format(self, document): ...
    def clone(self): ...

class PDFPatternIf:
    cond: Incomplete
    thenPart: Incomplete
    elsePart: Incomplete
    def __init__(self, cond, thenPart=[], elsePart=[]) -> None: ...



from __future__ import annotations

import sqlite3
from typing_extensions import assert_type


class MyConnection(sqlite3.Connection):
    pass


# Default return-type is Connection.
assert_type(sqlite3.connect(":memory:"), sqlite3.Connection)

# Providing an alternate factory changes the return-type.
assert_type(sqlite3.connect(":memory:", factory=MyConnection), MyConnection)

# Provides a true positive error. When checking the connect() function,
# mypy should report an arg-type error for the factory argument.
with sqlite3.connect(":memory:", factory=None) as con:  # type: ignore
    pass

# The Connection class also accepts a `factory` arg but it does not affect
# the return-type. This use case is not idiomatic--connections should be
# established using the `connect()` function, not directly (as shown here).
assert_type(sqlite3.Connection(":memory:", factory=None), sqlite3.Connection)
assert_type(sqlite3.Connection(":memory:", factory=MyConnection), sqlite3.Connection)



from typing import ClassVar
from typing_extensions import Self

import passlib.utils.handlers as uh

class _SHA2_Common(uh.HasManyBackends, uh.HasRounds, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    checksum_chars: ClassVar[str]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    implicit_rounds: bool
    def __init__(self, implicit_rounds: bool | None = None, **kwds) -> None: ...
    @classmethod
    def from_string(cls, hash: str | bytes) -> Self: ...  # type: ignore[override]
    backends: ClassVar[tuple[str, ...]]

class sha256_crypt(_SHA2_Common):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    default_rounds: ClassVar[int]

class sha512_crypt(_SHA2_Common):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    default_rounds: ClassVar[int]

__all__ = ["sha512_crypt", "sha256_crypt"]



from collections.abc import Iterable

from networkx.classes.graph import Graph, _Edge, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["betweenness_centrality_subset", "edge_betweenness_centrality_subset"]

@_dispatchable
def betweenness_centrality_subset(
    G: Graph[_Node],
    sources: Iterable[_Node],
    targets: Iterable[_Node],
    normalized: bool | None = False,
    weight: str | None = None,
) -> dict[_Node, float]: ...
@_dispatchable
def edge_betweenness_centrality_subset(
    G: Graph[_Node],
    sources: Iterable[_Node],
    targets: Iterable[_Node],
    normalized: bool | None = False,
    weight: str | None = None,
) -> dict[_Edge[_Node], float]: ...



from _typeshed import Incomplete

from ..rfc6749 import AuthorizationServer, ClientMixin
from ..rfc6749.requests import OAuth2Request

class JWTAuthenticationRequest:
    support_request: bool
    support_request_uri: bool
    def __init__(self, support_request: bool = True, support_request_uri: bool = True) -> None: ...
    def __call__(self, authorization_server: AuthorizationServer) -> None: ...
    def parse_authorization_request(self, authorization_server: AuthorizationServer, request: OAuth2Request) -> None: ...
    def get_request_object(self, request_uri: str): ...
    def resolve_client_public_keys(self, client: ClientMixin): ...
    def get_server_metadata(self) -> dict[str, Incomplete]: ...
    def get_client_require_signed_request_object(self, client: ClientMixin) -> bool: ...



from collections.abc import Iterable
from typing_extensions import Self

class VaultError(Exception):
    errors: Iterable[str] | str | None
    method: str | None
    url: str | None
    text: str | None
    json: object
    def __init__(
        self,
        message: str | None = None,
        errors: Iterable[str] | str | None = None,
        method: str | None = None,
        url: str | None = None,
        text: str | None = None,
        json: object | None = None,
    ) -> None: ...
    @classmethod
    def from_status(
        cls,
        status_code: int,
        message: str | None = ...,
        errors: Iterable[str] | str | None = ...,
        method: str | None = ...,
        url: str | None = ...,
        text: str | None = ...,
        json: object | None = ...,
    ) -> Self: ...

class InvalidRequest(VaultError): ...
class Unauthorized(VaultError): ...
class Forbidden(VaultError): ...
class InvalidPath(VaultError): ...
class UnsupportedOperation(VaultError): ...
class PreconditionFailed(VaultError): ...
class RateLimitExceeded(VaultError): ...
class InternalServerError(VaultError): ...
class VaultNotInitialized(VaultError): ...
class VaultDown(VaultError): ...
class UnexpectedError(VaultError): ...
class BadGateway(VaultError): ...
class ParamValidationError(VaultError): ...



from collections.abc import Callable, Collection

from authlib.oauth2 import OAuth2Request
from authlib.oauth2.rfc6749 import ClientMixin

__all__ = ["ClientAuthentication"]

class ClientAuthentication:
    query_client: Callable[[str], ClientMixin]
    def __init__(self, query_client: Callable[[str], ClientMixin]) -> None: ...
    def register(self, method: str, func: Callable[[Callable[[str], ClientMixin], OAuth2Request], ClientMixin]) -> None: ...
    def authenticate(self, request: OAuth2Request, methods: Collection[str], endpoint: str) -> ClientMixin: ...
    def __call__(self, request: OAuth2Request, methods: Collection[str], endpoint: str = "token") -> ClientMixin: ...



from collections.abc import Callable, Iterable, Sequence

import tensorflow as tf
from tensorflow._aliases import ShapeLike
from tensorflow.python.feature_column import feature_column_v2 as fc, sequence_feature_column as seq_fc

def numeric_column(
    key: str,
    shape: ShapeLike = (1,),
    default_value: float | None = None,
    dtype: tf.DType = ...,
    normalizer_fn: Callable[[tf.Tensor], tf.Tensor] | None = None,
) -> fc.NumericColumn: ...
def bucketized_column(source_column: fc.NumericColumn, boundaries: list[float] | tuple[float, ...]) -> fc.BucketizedColumn: ...
def embedding_column(
    categorical_column: fc.CategoricalColumn,
    dimension: int,
    combiner: fc._Combiners = "mean",
    initializer: Callable[[ShapeLike], tf.Tensor] | None = None,
    ckpt_to_load_from: str | None = None,
    tensor_name_in_ckpt: str | None = None,
    max_norm: float | None = None,
    trainable: bool = True,
    use_safe_embedding_lookup: bool = True,
) -> fc.EmbeddingColumn: ...
def shared_embeddings(
    categorical_columns: Iterable[fc.CategoricalColumn],
    dimension: int,
    combiner: fc._Combiners = "mean",
    initializer: Callable[[ShapeLike], tf.Tensor] | None = None,
    shared_embedding_collection_name: str | None = None,
    ckpt_to_load_from: str | None = None,
    tensor_name_in_ckpt: str | None = None,
    max_norm: float | None = None,
    trainable: bool = True,
    use_safe_embedding_lookup: bool = True,
) -> list[fc.SharedEmbeddingColumn]: ...
def categorical_column_with_identity(
    key: str, num_buckets: int, default_value: int | None = None
) -> fc.IdentityCategoricalColumn: ...
def categorical_column_with_hash_bucket(key: str, hash_bucket_size: int, dtype: tf.DType = ...) -> fc.HashedCategoricalColumn: ...
def categorical_column_with_vocabulary_file(
    key: str,
    vocabulary_file: str,
    vocabulary_size: int | None = None,
    dtype: tf.DType = ...,
    default_value: str | int | None = None,
    num_oov_buckets: int = 0,
    file_format: str | None = None,
) -> fc.VocabularyFileCategoricalColumn: ...
def categorical_column_with_vocabulary_list(
    key: str,
    vocabulary_list: Sequence[str] | Sequence[int],
    dtype: tf.DType | None = None,
    default_value: str | int | None = -1,
    num_oov_buckets: int = 0,
) -> fc.VocabularyListCategoricalColumn: ...
def indicator_column(categorical_column: fc.CategoricalColumn) -> fc.IndicatorColumn: ...
def weighted_categorical_column(
    categorical_column: fc.CategoricalColumn, weight_feature_key: str, dtype: tf.DType = ...
) -> fc.WeightedCategoricalColumn: ...
def crossed_column(
    keys: Iterable[str | fc.CategoricalColumn], hash_bucket_size: int, hash_key: int | None = None
) -> fc.CrossedColumn: ...
def sequence_numeric_column(
    key: str,
    shape: ShapeLike = (1,),
    default_value: float = 0.0,
    dtype: tf.DType = ...,
    normalizer_fn: Callable[[tf.Tensor], tf.Tensor] | None = None,
) -> seq_fc.SequenceNumericColumn: ...
def sequence_categorical_column_with_identity(
    key: str, num_buckets: int, default_value: int | None = None
) -> fc.SequenceCategoricalColumn: ...
def sequence_categorical_column_with_hash_bucket(
    key: str, hash_bucket_size: int, dtype: tf.DType = ...
) -> fc.SequenceCategoricalColumn: ...
def sequence_categorical_column_with_vocabulary_file(
    key: str,
    vocabulary_file: str,
    vocabulary_size: int | None = None,
    num_oov_buckets: int = 0,
    default_value: str | int | None = None,
    dtype: tf.DType = ...,
) -> fc.SequenceCategoricalColumn: ...
def sequence_categorical_column_with_vocabulary_list(
    key: str,
    vocabulary_list: Sequence[str] | Sequence[int],
    dtype: tf.DType | None = None,
    default_value: str | int | None = -1,
    num_oov_buckets: int = 0,
) -> fc.SequenceCategoricalColumn: ...
def make_parse_example_spec(
    feature_columns: Iterable[fc._FeatureColumn],
) -> dict[str, tf.io.FixedLenFeature | tf.io.VarLenFeature]: ...



from typing import Any, Literal, overload

from hvac.adapters import Adapter
from hvac.api import AuthMethods, SystemBackend
from hvac.api.secrets_engines import SecretsEngines
from requests import Session
from requests.models import Response

has_hcl_parser: bool

class Client:
    def __init__(
        self,
        url: str | None = None,
        token: str | None = None,
        cert: tuple[str, str] | None = None,
        verify: bool | str | None = None,
        timeout: int = 30,
        proxies: dict[str, str] | None = None,
        allow_redirects: bool = True,
        session: Session | None = None,
        adapter: type[Adapter[Any]] = ...,
        namespace: str | None = None,
        **kwargs: Any,
    ) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    @property
    def adapter(self) -> Adapter[Any]: ...
    @adapter.setter
    def adapter(self, adapter: Adapter[Any]) -> None: ...
    @property
    def url(self) -> str: ...
    @url.setter
    def url(self, url: str) -> None: ...
    @property
    def token(self) -> str: ...
    @token.setter
    def token(self, token: str) -> None: ...
    @property
    def session(self) -> Session: ...
    @session.setter
    def session(self, session: Session) -> None: ...
    @property
    def allow_redirects(self) -> bool: ...
    @allow_redirects.setter
    def allow_redirects(self, allow_redirects: bool) -> None: ...
    @property
    def auth(self) -> AuthMethods: ...
    @property
    def secrets(self) -> SecretsEngines: ...
    @property
    def sys(self) -> SystemBackend: ...
    @property
    def generate_root_status(self) -> dict[str, Any] | Response: ...
    @property
    def key_status(self) -> dict[str, Any] | Response: ...
    @property
    def rekey_status(self) -> dict[str, Any] | Response: ...
    @property
    def ha_status(self) -> dict[str, Any] | Response: ...
    @property
    def seal_status(self) -> dict[str, Any] | Response: ...
    def read(self, path: str, wrap_ttl: int | str | None = None) -> dict[str, Any] | Response | None: ...
    def list(self, path: str) -> dict[str, Any] | Response | None: ...
    def write(self, path: str, wrap_ttl: int | str | None, **kwargs: Any) -> dict[str, Any] | Response: ...
    def write_data(
        self, path: str, *, data: dict[str, Any] | None = None, wrap_ttl: int | str | None = None
    ) -> dict[str, Any] | Response: ...
    def delete(self, path: str) -> None: ...
    @overload
    def get_policy(self, name: str, parse: Literal[False] = False) -> str | None: ...
    @overload
    def get_policy(self, name: str, parse: Literal[True]) -> dict[str, Any] | None: ...
    def lookup_token(
        self, token: str | None = None, accessor: bool = False, wrap_ttl: int | str | None = None
    ) -> dict[str, Any] | Response: ...
    def revoke_token(self, token: str, orphan: bool = False, accessor: bool = False) -> None: ...
    def renew_token(
        self, token: str, increment: bool | None = None, wrap_ttl: int | str | None = None
    ) -> dict[str, Any] | Response: ...
    def logout(self, revoke_token: bool = False) -> None: ...
    def is_authenticated(self) -> bool: ...
    def auth_cubbyhole(self, token: str) -> Response: ...
    def login(self, url: str, use_token: bool = True, **kwargs: Any) -> Response: ...



from typing import TypeVar

_T = TypeVar("_T")

def get_all(type_obj: type[_T], include_subtypes: bool = True) -> list[_T]: ...

class GCToggler:
    postcollect: bool
    def __init__(self, postcollect: bool = False) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, exc_type, exc_val, exc_tb) -> None: ...

toggle_gc: GCToggler
toggle_gc_postcollect: GCToggler

__all__ = ["get_all", "GCToggler", "toggle_gc", "toggle_gc_postcollect"]



import argparse
import enum
from _typeshed import Incomplete
from collections.abc import Generator, Sequence

from .formatting.base import BaseFormatter
from .statistics import Statistics

__all__ = ("StyleGuide",)

class Selected(enum.Enum):
    Explicitly = "explicitly selected"
    Implicitly = "implicitly selected"

class Ignored(enum.Enum):
    Explicitly = "explicitly ignored"
    Implicitly = "implicitly ignored"

class Decision(enum.Enum):
    Ignored = "ignored error"
    Selected = "selected error"

class DecisionEngine:
    cache: Incomplete
    selected_explicitly: Incomplete
    ignored_explicitly: Incomplete
    selected: Incomplete
    ignored: Incomplete
    def __init__(self, options: argparse.Namespace) -> None: ...
    def was_selected(self, code: str) -> Selected | Ignored: ...
    def was_ignored(self, code: str) -> Selected | Ignored: ...
    def make_decision(self, code: str) -> Decision: ...
    def decision_for(self, code: str) -> Decision: ...

class StyleGuideManager:
    options: Incomplete
    formatter: Incomplete
    stats: Incomplete
    decider: Incomplete
    style_guides: Incomplete
    default_style_guide: Incomplete
    style_guide_for: Incomplete
    def __init__(self, options: argparse.Namespace, formatter: BaseFormatter, decider: DecisionEngine | None = None) -> None: ...
    def populate_style_guides_with(self, options: argparse.Namespace) -> Generator[StyleGuide, None, None]: ...
    def processing_file(self, filename: str) -> Generator[StyleGuide, None, None]: ...
    def handle_error(
        self, code: str, filename: str, line_number: int, column_number: int, text: str, physical_line: str | None = None
    ) -> int: ...

class StyleGuide:
    options: Incomplete
    formatter: Incomplete
    stats: Incomplete
    decider: Incomplete
    filename: Incomplete
    def __init__(
        self,
        options: argparse.Namespace,
        formatter: BaseFormatter,
        stats: Statistics,
        filename: str | None = None,
        decider: DecisionEngine | None = None,
    ) -> None: ...
    def copy(self, filename: str | None = None, extend_ignore_with: Sequence[str] | None = None) -> StyleGuide: ...
    def processing_file(self, filename: str) -> Generator[StyleGuide, None, None]: ...
    def applies_to(self, filename: str) -> bool: ...
    def should_report_error(self, code: str) -> Decision: ...
    def handle_error(
        self, code: str, filename: str, line_number: int, column_number: int, text: str, physical_line: str | None = None
    ) -> int: ...



from collections.abc import Generator

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = ["number_weakly_connected_components", "weakly_connected_components", "is_weakly_connected"]

@_dispatchable
def weakly_connected_components(G: DiGraph[_Node]) -> Generator[set[_Node]]: ...
@_dispatchable
def number_weakly_connected_components(G: DiGraph[_Node]) -> int: ...
@_dispatchable
def is_weakly_connected(G: DiGraph[_Node]) -> bool: ...



import argparse
from _typeshed import Unused
from typing import Any

from ..formatting import base as formatter
from ..main import application as app

__all__ = ("get_style_guide",)

class Report:
    def __init__(self, application: app.Application) -> None: ...
    @property
    def total_errors(self) -> int: ...
    def get_statistics(self, violation: str) -> list[str]: ...

class StyleGuide:
    def __init__(self, application: app.Application) -> None: ...
    @property
    def options(self) -> argparse.Namespace: ...
    @property
    def paths(self) -> list[str]: ...
    def check_files(self, paths: list[str] | None = None) -> Report: ...
    def excluded(self, filename: str, parent: str | None = None) -> bool: ...
    def init_report(self, reporter: type[formatter.BaseFormatter] | None = None) -> None: ...
    def input_file(self, filename: str, lines: Unused = None, expected: Unused = None, line_offset: Unused = 0) -> Report: ...

def get_style_guide(**kwargs: Any) -> StyleGuide: ...



"""Standardized transformation process and error interface."""

from __future__ import annotations

import time
from abc import ABC, abstractmethod
from threading import Event
from typing import TYPE_CHECKING, Generic, TypeVar

from jaclang.pycore.codeinfo import CodeLocInfo
from jaclang.pycore.log import logging
from jaclang.pycore.unitree import UniNode

if TYPE_CHECKING:
    from jaclang.pycore.program import JacProgram

# Type variables for transforms - T_in/T_out are unconstrained for base transforms
T_in = TypeVar("T_in")
T_out = TypeVar("T_out")

# Type variables for AST transforms - constrained to UniNode
T = TypeVar("T", bound=UniNode)
R = TypeVar("R", bound=UniNode)


class Alert:
    """Alert interface."""

    def __init__(self, msg: str, loc: CodeLocInfo, from_pass: type[Transform]) -> None:
        """Initialize alert."""
        self.msg = msg
        self.loc: CodeLocInfo = loc
        self.from_pass: type[Transform] = from_pass

    def __str__(self) -> str:
        """Return string representation of alert."""
        return f"{self.loc.mod_path}, line {self.loc.first_line}, col {self.loc.col_start}: {self.msg}"

    def __repr__(self) -> str:
        """Return string representation of alert."""
        return self.as_log()

    def as_log(self, *, colors: bool = False) -> str:
        """Return the alert as a single line log as opposed to the pretty print."""
        from jaclang.pycore.helpers import ANSIColors

        file_path: str = self.loc.mod_path
        if file_path == "":
            return self.msg  # There are error messages without file references.

        line: int = self.loc.first_line
        column: int = self.loc.col_start

        # TODO: Set if the alert is error or warning and color accordingly.
        msg = self.msg if not colors else f"{ANSIColors.RED}{self.msg}{ANSIColors.END}"
        return f"{file_path}:{line}:{column} {msg}"

    def pretty_print(self, *, colors: bool = False) -> str:
        """Pretty prints the Alert to show the alert with source location."""
        from jaclang.pycore.helpers import pretty_print_source_location

        pretty_dump = pretty_print_source_location(
            self.loc.mod_path,
            self.loc.orig_src.code,
            self.loc.first_line,
            self.loc.pos_start,
            self.loc.pos_end,
            colors=colors,
        )
        if pretty_dump != "":
            pretty_dump = "\n" + pretty_dump
        return self.as_log(colors=colors) + pretty_dump


class BaseTransform(ABC, Generic[T_in, T_out]):
    """Base abstract class for all transforms (no UniNode constraint).

    Use this base class for transforms that don't operate on AST nodes,
    such as parser transforms that take string input and produce parse trees.
    """

    def __init__(
        self, ir_in: T_in, prog: JacProgram, cancel_token: Event | None = None
    ) -> None:
        """Initialize pass."""
        self.logger = logging.getLogger(self.__class__.__name__)
        self.errors_had: list[Alert] = []
        self.warnings_had: list[Alert] = []
        self.prog = prog
        self.time_taken = 0.0
        self.ir_in: T_in = ir_in
        self.cancel_token = cancel_token
        self.pre_transform()
        self.ir_out: T_out = self.timed_transform(ir_in=ir_in)
        self.post_transform()

    def timed_transform(
        self,
        ir_in: T_in,
    ) -> T_out:
        """Transform with time tracking."""
        start_time = time.time()
        ir_out = self.transform(ir_in=ir_in)
        self.time_taken = time.time() - start_time
        return ir_out

    def pre_transform(self) -> None:
        """Pre-transform hook."""
        pass

    def post_transform(self) -> None:
        """Post-transform hook."""
        pass

    @abstractmethod
    def transform(self, ir_in: T_in) -> T_out:
        """Transform interface."""
        pass

    def log_info(self, msg: str) -> None:
        """Log info."""
        self.logger.info(msg)

    def is_canceled(self) -> bool:
        """Check if the pass has been canceled."""
        return self.cancel_token is not None and self.cancel_token.is_set()


class Transform(BaseTransform[T, R]):
    """Abstract class for IR passes that operate on AST nodes.

    This class extends BaseTransform with UniNode-specific functionality
    like error/warning logging with source location tracking.
    """

    def __init__(
        self, ir_in: T, prog: JacProgram, cancel_token: Event | None = None
    ) -> None:
        """Initialize pass."""
        self.cur_node: UniNode = ir_in  # tracks current node during traversal
        super().__init__(ir_in, prog, cancel_token)

    def log_error(self, msg: str, node_override: UniNode | None = None) -> None:
        """Pass Error."""
        alrt = Alert(
            msg,
            self.cur_node.loc if not node_override else node_override.loc,
            self.__class__,
        )
        self.errors_had.append(alrt)
        self.prog.errors_had.append(alrt)
        # self.logger.error(alrt.as_log())

    def log_warning(self, msg: str, node_override: UniNode | None = None) -> None:
        """Pass Error."""
        alrt = Alert(
            msg,
            self.cur_node.loc if not node_override else node_override.loc,
            self.__class__,
        )
        self.warnings_had.append(alrt)
        self.prog.warnings_had.append(alrt)
        # self.logger.warning(alrt.as_log())

    def ice(self, msg: str = "Something went horribly wrong!") -> RuntimeError:
        """Pass Error."""
        self.log_error(f"ICE: Pass {self.__class__.__name__} - {msg}")
        return RuntimeError(
            f"Internal Compiler Error: Pass {self.__class__.__name__} - {msg}"
        )



import array
import datetime
import io
import logging
import traceback
from collections.abc import Sequence
from typing import IO, AnyStr, Generic
from typing_extensions import Self, TypeAlias

__date__: str
__version__: str
__author__: str

__all__ = [
    "isOleFile",
    "OleFileIO",
    "OleMetadata",
    "enable_logging",
    "MAGIC",
    "STGTY_EMPTY",
    "STGTY_STREAM",
    "STGTY_STORAGE",
    "STGTY_ROOT",
    "STGTY_PROPERTY",
    "STGTY_LOCKBYTES",
    "MINIMAL_OLEFILE_SIZE",
    "DEFECT_UNSURE",
    "DEFECT_POTENTIAL",
    "DEFECT_INCORRECT",
    "DEFECT_FATAL",
    "DEFAULT_PATH_ENCODING",
    "MAXREGSECT",
    "DIFSECT",
    "FATSECT",
    "ENDOFCHAIN",
    "FREESECT",
    "MAXREGSID",
    "NOSTREAM",
    "UNKNOWN_SIZE",
    "WORD_CLSID",
    "OleFileIONotClosed",
]

UINT32: str

DEFAULT_PATH_ENCODING: str | None

def get_logger(name: str, level: int = 51) -> logging.Logger: ...

log: logging.Logger

def enable_logging() -> None: ...

MAGIC: bytes

MAXREGSECT: int
DIFSECT: int
FATSECT: int
ENDOFCHAIN: int
FREESECT: int

MAXREGSID: int
NOSTREAM: int

STGTY_EMPTY: int
STGTY_STORAGE: int
STGTY_STREAM: int
STGTY_LOCKBYTES: int
STGTY_PROPERTY: int
STGTY_ROOT: int

UNKNOWN_SIZE: int

VT_EMPTY: int
VT_NULL: int
VT_I2: int
VT_I4: int
VT_R4: int
VT_R8: int
VT_CY: int
VT_DATE: int
VT_BSTR: int
VT_DISPATCH: int
VT_ERROR: int
VT_BOOL: int
VT_VARIANT: int
VT_UNKNOWN: int
VT_DECIMAL: int
VT_I1: int
VT_UI1: int
VT_UI2: int
VT_UI4: int
VT_I8: int
VT_UI8: int
VT_INT: int
VT_UINT: int
VT_VOID: int
VT_HRESULT: int
VT_PTR: int
VT_SAFEARRAY: int
VT_CARRAY: int
VT_USERDEFINED: int
VT_LPSTR: int
VT_LPWSTR: int
VT_FILETIME: int
VT_BLOB: int
VT_STREAM: int
VT_STORAGE: int
VT_STREAMED_OBJECT: int
VT_STORED_OBJECT: int
VT_BLOB_OBJECT: int
VT_CF: int
VT_CLSID: int
VT_VECTOR: int

VT: dict[int, str]

WORD_CLSID: str
DEFECT_UNSURE: int
DEFECT_POTENTIAL: int
DEFECT_INCORRECT: int
DEFECT_FATAL: int
MINIMAL_OLEFILE_SIZE: int

def isOleFile(filename: IO[bytes] | bytes | str | None = None, data: bytes | None = None) -> bool: ...
def i8(c: bytes | int) -> int: ...
def i16(c: bytes, o: int = 0) -> int: ...
def i32(c: bytes, o: int = 0) -> int: ...
def _clsid(clsid: bytes) -> str: ...
def filetime2datetime(filetime: int) -> datetime.datetime: ...

class OleFileError(IOError): ...
class NotOleFileError(OleFileError): ...

class OleMetadata:
    SUMMARY_ATTRIBS: list[str]
    DOCSUM_ATTRIBS: list[str]

    def __init__(self) -> None: ...
    def parse_properties(self, ole_file: OleFileIO[AnyStr]) -> None: ...
    def dump(self) -> None: ...

class OleFileIONotClosed(RuntimeWarning):
    def __init__(self, stack_of_open: traceback.FrameSummary | None = None) -> None: ...

class OleStream(io.BytesIO):
    def __init__(
        self,
        fp: IO[bytes],
        sect: int,
        size: int,
        offset: int,
        sectorsize: int,
        fat: list[int],
        filesize: int,
        olefileio: OleFileIO[AnyStr],
    ) -> None: ...

class OleDirectoryEntry(Generic[AnyStr]):
    STRUCT_DIRENTRY: str
    DIRENTRY_SIZE: int
    clsid: str

    def __init__(self, entry: bytes, sid: int, ole_file: OleFileIO[AnyStr]) -> None: ...
    def build_sect_chain(self, ole_file: OleFileIO[AnyStr]) -> None: ...
    def build_storage_tree(self) -> None: ...
    def append_kids(self, child_sid: int) -> None: ...
    def __eq__(self, other: OleDirectoryEntry[AnyStr]) -> bool: ...  # type: ignore[override]
    def __lt__(self, other: OleDirectoryEntry[AnyStr]) -> bool: ...
    def __ne__(self, other: OleDirectoryEntry[AnyStr]) -> bool: ...  # type: ignore[override]
    def __le__(self, other: OleDirectoryEntry[AnyStr]) -> bool: ...
    def dump(self, tab: int = 0) -> None: ...
    def getmtime(self) -> datetime.datetime | None: ...
    def getctime(self) -> datetime.datetime | None: ...

_Property: TypeAlias = int | str | bytes | bool | None

class OleFileIO(Generic[AnyStr]):
    root: OleDirectoryEntry[AnyStr] | None

    def __init__(
        self,
        filename: IO[bytes] | AnyStr | None = None,
        raise_defects: int = 40,
        write_mode: bool = False,
        debug: bool = False,
        path_encoding: str | None = DEFAULT_PATH_ENCODING,  # noqa: Y011
    ) -> None: ...
    def __del__(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...
    def _raise_defect(
        self, defect_level: int, message: str, exception_type: type[Exception] = OleFileError  # noqa: Y011
    ) -> None: ...
    def _decode_utf16_str(self, utf16_str: bytes, errors: str = "replace") -> str | bytes: ...
    def open(self, filename: IO[bytes] | AnyStr, write_mode: bool = False) -> None: ...
    def close(self) -> None: ...
    def _close(self, warn: bool = False) -> None: ...
    def _check_duplicate_stream(self, first_sect: int, minifat: bool = False) -> None: ...
    def dumpfat(self, fat: Sequence[int], firstindex: int = 0) -> None: ...
    def dumpsect(self, sector: bytes, firstindex: int = 0) -> None: ...
    def sect2array(self, sect: bytes) -> Sequence[int]: ...
    def loadfat_sect(self, sect: bytes | array.array[int]) -> int | None: ...
    def loadfat(self, header: bytes) -> None: ...
    def loadminifat(self) -> None: ...
    def getsect(self, sect: int) -> bytes: ...
    def write_sect(self, sect: int, data: bytes, padding: bytes = b"\x00") -> None: ...
    def _write_mini_sect(self, fp_pos: int, data: bytes, padding: bytes = b"\x00") -> None: ...
    def loaddirectory(self, sect: int) -> None: ...
    def _load_direntry(self, sid: int) -> OleDirectoryEntry[AnyStr]: ...
    def dumpdirectory(self) -> None: ...
    def _open(self, start: int, size: int = 0x7FFFFFFF, force_FAT: bool = False) -> OleStream: ...
    def _list(
        self,
        files: list[list[AnyStr]],
        prefix: list[AnyStr],
        node: OleDirectoryEntry[AnyStr],
        streams: bool = True,
        storages: bool = False,
    ) -> None: ...
    def listdir(self, streams: bool = True, storages: bool = False) -> list[list[AnyStr]]: ...
    def _find(self, filename: str | Sequence[str]) -> int: ...
    def openstream(self, filename: AnyStr | Sequence[AnyStr]) -> OleStream: ...
    def _write_mini_stream(self, entry: OleDirectoryEntry[AnyStr], data_to_write: bytes) -> None: ...
    def write_stream(self, stream_name: str | Sequence[str], data: bytes) -> None: ...
    def get_type(self, filename: AnyStr | Sequence[AnyStr]) -> bool | int: ...
    def getclsid(self, filename: AnyStr | Sequence[AnyStr]) -> str: ...
    def getmtime(self, filename: AnyStr | Sequence[AnyStr]) -> datetime.datetime | None: ...
    def getctime(self, filename: AnyStr | Sequence[AnyStr]) -> datetime.datetime | None: ...
    def exists(self, filename: AnyStr | Sequence[AnyStr]) -> bool: ...
    def get_size(self, filename: AnyStr | Sequence[AnyStr]) -> int: ...
    def get_rootentry_name(self) -> bytes: ...
    def getproperties(
        self, filename: AnyStr | Sequence[AnyStr], convert_time: bool = False, no_conversion: list[int] | None = None
    ) -> dict[int, list[_Property] | _Property]: ...
    def _parse_property(
        self, s: bytes, offset: int, property_id: int, property_type: int, convert_time: bool, no_conversion: list[int]
    ) -> list[_Property] | _Property: ...
    def _parse_property_basic(
        self, s: bytes, offset: int, property_id: int, property_type: int, convert_time: bool, no_conversion: list[int]
    ) -> tuple[_Property, int]: ...
    def get_metadata(self) -> OleMetadata: ...
    def get_userdefined_properties(
        self, filename: AnyStr | Sequence[AnyStr], convert_time: bool = False, no_conversion: list[int] | None = None
    ) -> list[dict[str, bytes | int | None]]: ...

def main() -> None: ...



from . import items as items
from .console_menu import ConsoleMenu as ConsoleMenu, Screen as Screen, clear_terminal as clear_terminal
from .menu_formatter import MenuFormatBuilder as MenuFormatBuilder
from .multiselect_menu import MultiSelectMenu as MultiSelectMenu
from .prompt_utils import PromptUtils as PromptUtils
from .selection_menu import SelectionMenu as SelectionMenu

__all__ = [
    "ConsoleMenu",
    "SelectionMenu",
    "MultiSelectMenu",
    "MenuFormatBuilder",
    "PromptUtils",
    "Screen",
    "items",
    "clear_terminal",
]



from _typeshed import Incomplete

CONTENT_TYPE_FORM_URLENCODED: str
CONTENT_TYPE_MULTI_PART: str

class ClientAuth:
    SIGNATURE_METHODS: Incomplete
    @classmethod
    def register_signature_method(cls, name, sign) -> None: ...
    client_id: Incomplete
    client_secret: Incomplete
    token: Incomplete
    token_secret: Incomplete
    redirect_uri: Incomplete
    signature_method: Incomplete
    signature_type: Incomplete
    rsa_key: Incomplete
    verifier: Incomplete
    realm: Incomplete
    force_include_body: Incomplete
    def __init__(
        self,
        client_id,
        client_secret=None,
        token=None,
        token_secret=None,
        redirect_uri=None,
        rsa_key=None,
        verifier=None,
        signature_method="HMAC-SHA1",
        signature_type="HEADER",
        realm=None,
        force_include_body: bool = False,
    ) -> None: ...
    def get_oauth_signature(self, method, uri, headers, body): ...
    def get_oauth_params(self, nonce, timestamp): ...
    def sign(self, method, uri, headers, body): ...
    def prepare(self, method, uri, headers, body): ...

def generate_nonce(): ...
def generate_timestamp(): ...



from _typeshed import Incomplete
from typing_extensions import Self

from braintree.validation_error import ValidationError

class ValidationErrorCollection:
    data: dict[str, Incomplete]
    def __init__(self, data: dict[str, Incomplete] | None = None) -> None: ...
    @property
    def deep_errors(self) -> list[ValidationError]: ...
    def for_index(self, index: int | str) -> Self: ...
    def for_object(self, nested_key: str) -> Self: ...
    def on(self, attribute: str) -> list[ValidationError]: ...
    @property
    def deep_size(self) -> int: ...
    @property
    def errors(self) -> list[ValidationError]: ...
    @property
    def size(self) -> int: ...
    def __getitem__(self, index: int) -> ValidationError: ...
    def __len__(self) -> int: ...



import logging
from collections.abc import Callable
from typing import Any, TypeVar
from typing_extensions import TypeAlias

import click

_AnyCallable: TypeAlias = Callable[..., Any]
_FC = TypeVar("_FC", bound=_AnyCallable | click.Command)

def simple_verbosity_option(logger: logging.Logger | str | None = None, *names: str, **kwargs: Any) -> Callable[[_FC], _FC]: ...



import ast
import logging
from collections.abc import Generator
from typing import Any, ClassVar

logger: logging.Logger

class Visitor(ast.NodeVisitor):
    def __init__(self) -> None: ...
    def visit_Assign(self, node: ast.Assign) -> None: ...
    def visit_Call(self, node: ast.Call) -> None: ...
    def visit_With(self, node: ast.With) -> None: ...
    def visit_Expr(self, node: ast.Expr) -> None: ...
    def visit_BoolOp(self, node: ast.BoolOp) -> None: ...
    def visit_If(self, node: ast.If) -> None: ...
    def visit_For(self, node: ast.For) -> None: ...
    def visit_Subscript(self, node: ast.Subscript) -> None: ...
    def visit_Try(self, node: ast.Try) -> None: ...
    def visit_UnaryOp(self, node_v: ast.UnaryOp) -> None: ...
    def visit_IfExp(self, node: ast.IfExp) -> None: ...
    def visit_Compare(self, node: ast.Compare) -> None: ...
    def visit_ClassDef(self, node: ast.ClassDef) -> None: ...

class Plugin:
    name: ClassVar[str]
    version: ClassVar[str]
    def __init__(self, tree: ast.AST) -> None: ...
    def run(self) -> Generator[tuple[int, int, str, type[Any]], None, None]: ...

def add_meta(root: ast.AST, level: int = 0) -> None: ...



from _typeshed import Incomplete

class JWSAlgorithm:
    name: Incomplete
    description: Incomplete
    algorithm_type: str
    algorithm_location: str
    def prepare_key(self, raw_data) -> None: ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

class JWSHeader(dict[str, object]):
    protected: Incomplete
    header: Incomplete
    def __init__(self, protected, header) -> None: ...
    @classmethod
    def from_dict(cls, obj): ...

class JWSObject(dict[str, object]):
    header: Incomplete
    payload: Incomplete
    type: Incomplete
    def __init__(self, header, payload, type: str = "compact") -> None: ...
    @property
    def headers(self): ...



from _typeshed import ReadableBuffer, StrOrBytesPath, SupportsWrite, WriteableBuffer
from collections.abc import Mapping
from compression._common import _streams
from compression.zstd import ZstdDict
from io import TextIOWrapper, _WrappedBuffer
from typing import Literal, Protocol, overload, type_check_only
from typing_extensions import TypeAlias

from _zstd import ZstdCompressor, _ZstdCompressorFlushBlock, _ZstdCompressorFlushFrame

__all__ = ("ZstdFile", "open")

_ReadBinaryMode: TypeAlias = Literal["r", "rb"]
_WriteBinaryMode: TypeAlias = Literal["w", "wb", "x", "xb", "a", "ab"]
_ReadTextMode: TypeAlias = Literal["rt"]
_WriteTextMode: TypeAlias = Literal["wt", "xt", "at"]

@type_check_only
class _FileBinaryRead(_streams._Reader, Protocol):
    def close(self) -> None: ...

@type_check_only
class _FileBinaryWrite(SupportsWrite[bytes], Protocol):
    def close(self) -> None: ...

class ZstdFile(_streams.BaseStream):
    FLUSH_BLOCK = ZstdCompressor.FLUSH_BLOCK
    FLUSH_FRAME = ZstdCompressor.FLUSH_FRAME

    @overload
    def __init__(
        self,
        file: StrOrBytesPath | _FileBinaryRead,
        /,
        mode: _ReadBinaryMode = "r",
        *,
        level: None = None,
        options: Mapping[int, int] | None = None,
        zstd_dict: ZstdDict | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        file: StrOrBytesPath | _FileBinaryWrite,
        /,
        mode: _WriteBinaryMode,
        *,
        level: int | None = None,
        options: Mapping[int, int] | None = None,
        zstd_dict: ZstdDict | None = None,
    ) -> None: ...
    def write(self, data: ReadableBuffer, /) -> int: ...
    def flush(self, mode: _ZstdCompressorFlushBlock | _ZstdCompressorFlushFrame = 1) -> bytes: ...  # type: ignore[override]
    def read(self, size: int | None = -1) -> bytes: ...
    def read1(self, size: int | None = -1) -> bytes: ...
    def readinto(self, b: WriteableBuffer) -> int: ...
    def readinto1(self, b: WriteableBuffer) -> int: ...
    def readline(self, size: int | None = -1) -> bytes: ...
    def seek(self, offset: int, whence: int = 0) -> int: ...
    def peek(self, size: int = -1) -> bytes: ...
    @property
    def name(self) -> str | bytes: ...
    @property
    def mode(self) -> Literal["rb", "wb"]: ...

@overload
def open(
    file: StrOrBytesPath | _FileBinaryRead,
    /,
    mode: _ReadBinaryMode = "rb",
    *,
    level: None = None,
    options: Mapping[int, int] | None = None,
    zstd_dict: ZstdDict | None = None,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
) -> ZstdFile: ...
@overload
def open(
    file: StrOrBytesPath | _FileBinaryWrite,
    /,
    mode: _WriteBinaryMode,
    *,
    level: int | None = None,
    options: Mapping[int, int] | None = None,
    zstd_dict: ZstdDict | None = None,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
) -> ZstdFile: ...
@overload
def open(
    file: StrOrBytesPath | _WrappedBuffer,
    /,
    mode: _ReadTextMode,
    *,
    level: None = None,
    options: Mapping[int, int] | None = None,
    zstd_dict: ZstdDict | None = None,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
) -> TextIOWrapper: ...
@overload
def open(
    file: StrOrBytesPath | _WrappedBuffer,
    /,
    mode: _WriteTextMode,
    *,
    level: int | None = None,
    options: Mapping[int, int] | None = None,
    zstd_dict: ZstdDict | None = None,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
) -> TextIOWrapper: ...



from collections.abc import Callable
from typing_extensions import Self

import tensorflow as tf
from tensorflow._aliases import ShapeLike
from tensorflow.python.feature_column.feature_column_v2 import SequenceDenseColumn, _ExampleSpec, _FeatureColumn

# Strangely at runtime most of Sequence feature columns are defined in feature_column_v2 except
# for this one.
class SequenceNumericColumn(SequenceDenseColumn):
    key: str
    shape: ShapeLike
    default_value: float
    dtype: tf.DType
    normalizer_fn: Callable[[tf.Tensor], tf.Tensor] | None

    def __new__(
        _cls,
        key: str,
        shape: ShapeLike,
        default_value: float,
        dtype: tf.DType,
        normalizer_fn: Callable[[tf.Tensor], tf.Tensor] | None,
    ) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...



"""Setup script with custom build hook for parser generation."""

import os
import sys

from setuptools import setup
from setuptools.command.build_py import build_py

# Add source directory to path for build-time imports
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))


class BuildPyWithParser(build_py):
    """Custom build_py command that generates parsers before building."""

    def run(self) -> None:
        """Generate static parsers, then run the standard build_py."""
        from jaclang.compiler import gen_all_parsers

        gen_all_parsers()
        super().run()


setup(cmdclass={"build_py": BuildPyWithParser})



from collections.abc import Callable
from typing import Final

certifi_available: bool
certifi_where: Callable[[], str] | None
custom_ca_locater_available: bool
custom_ca_locater_where: Callable[..., str] | None
BUILTIN_CA_CERTS: Final[str]

def where() -> str: ...



from collections.abc import Callable
from logging import Logger

from oauthlib.common import Request, _HTTPMethod
from oauthlib.oauth2.rfc6749.endpoints.base import BaseEndpoint
from oauthlib.openid.connect.core.request_validator import RequestValidator

log: Logger

class DeviceAuthorizationEndpoint(BaseEndpoint):
    request_validator: RequestValidator
    user_code_generator: Callable[[None], str] | None
    def __init__(
        self,
        request_validator: RequestValidator,
        verification_uri: str,
        expires_in: int = 1800,
        interval: int | None = None,
        verification_uri_complete: str | None = None,
        user_code_generator: Callable[[None], str] | None = None,
    ) -> None: ...
    @property
    def interval(self) -> int | None: ...
    @property
    def expires_in(self) -> int: ...
    @property
    def verification_uri(self) -> str: ...
    def verification_uri_complete(self, user_code: str) -> str | None: ...
    def validate_device_authorization_request(self, request: Request) -> None: ...
    def create_device_authorization_response(
        self, uri: str, http_method: _HTTPMethod = "POST", body: str | None = None, headers: dict[str, str] | None = None
    ) -> tuple[dict[str, str], dict[str, str | int], int]: ...



from collections.abc import Hashable
from functools import cached_property
from typing import Any, ClassVar, overload
from typing_extensions import TypeAlias, TypeVar

from networkx.classes.coreviews import MultiAdjacencyView
from networkx.classes.graph import Graph, _MapFactory, _Node
from networkx.classes.multidigraph import MultiDiGraph
from networkx.classes.reportviews import DiMultiDegreeView, MultiDegreeView, MultiEdgeView, OutMultiEdgeView

_MultiEdge: TypeAlias = tuple[_Node, _Node, int]  # noqa: Y047

_DefaultT = TypeVar("_DefaultT")
_KeyT = TypeVar("_KeyT", bound=Hashable)

__all__ = ["MultiGraph"]

# NOTE: Graph subclasses relationships are so complex
# we're only overriding methods that differ in signature from the base classes
# to use inheritance to our advantage and reduce complexity
class MultiGraph(Graph[_Node]):
    edge_key_dict_factory: ClassVar[_MapFactory]
    def to_directed_class(self) -> type[MultiDiGraph[_Node]]: ...
    def to_undirected_class(self) -> type[MultiGraph[_Node]]: ...
    def __init__(self, incoming_graph_data=None, multigraph_input: bool | None = None, **attr: Any) -> None: ...
    @cached_property
    def adj(self) -> MultiAdjacencyView[_Node, _Node, dict[str, Any]]: ...  # data can be any type
    def new_edge_key(self, u: _Node, v: _Node) -> int: ...
    # key : hashable identifier, optional (default=lowest unused integer)
    @overload  # type: ignore[override] # More complex overload
    def add_edge(self, u_for_edge: _Node, v_for_edge: _Node, key: int | None = None, **attr: Any) -> int: ...
    @overload
    def add_edge(self, u_for_edge: _Node, v_for_edge: _Node, key: _KeyT, **attr: Any) -> _KeyT: ...
    def remove_edge(self, u: _Node, v: _Node, key: Hashable | None = None) -> None: ...
    def has_edge(self, u: _Node, v: _Node, key: Hashable | None = None) -> bool: ...
    @cached_property
    # Including subtypes' possible return types for LSP
    def edges(self) -> MultiEdgeView[_Node] | OutMultiEdgeView[_Node]: ...
    # key : hashable identifier, optional (default=None).
    # default : any Python object (default=None). Value to return if the specific edge (u, v, key) is not found.
    # Returns: The edge attribute dictionary.
    @overload  # type: ignore[override]
    def get_edge_data(
        self, u: _Node, v: _Node, key: Hashable, default: _DefaultT | None = None
    ) -> dict[str, Any] | _DefaultT: ...
    # default : any Python object (default=None). Value to return if there are no edges between u and v and no key is specified.
    # Returns: A dictionary mapping edge keys to attribute dictionaries for each of those edges if no specific key is provided.
    @overload
    def get_edge_data(
        self, u: _Node, v: _Node, key: None = None, default: _DefaultT | None = None
    ) -> dict[Hashable, dict[str, Any] | _DefaultT]: ...
    def copy(self, as_view: bool = False) -> MultiGraph[_Node]: ...
    @cached_property
    # Including subtypes' possible return types for LSP
    def degree(self) -> MultiDegreeView[_Node] | DiMultiDegreeView[_Node]: ...
    def to_directed(self, as_view: bool = False) -> MultiDiGraph[_Node]: ...
    def to_undirected(self, as_view: bool = False) -> MultiGraph[_Node]: ...



import abc
from decimal import Decimal
from typing import Any, NamedTuple
from xml.etree.ElementTree import Element, QName

from ...._types import Box
from ...svg import SvgFragmentImage, SvgPathImage
from .base import QRModuleDrawer

ANTIALIASING_FACTOR: int

class Coords(NamedTuple):
    x0: Decimal
    y0: Decimal
    x1: Decimal
    y1: Decimal
    xh: Decimal
    yh: Decimal

class BaseSvgQRModuleDrawer(QRModuleDrawer, metaclass=abc.ABCMeta):
    img: SvgFragmentImage
    size_ratio: Decimal
    # kwargs are used to allow for subclasses with additional keyword arguments
    def __init__(self, *, size_ratio: Decimal = ..., **kwargs: Any) -> None: ...
    box_delta: float
    box_size: Decimal
    box_half: Decimal
    def coords(self, box: Box) -> Coords: ...

class SvgQRModuleDrawer(BaseSvgQRModuleDrawer, metaclass=abc.ABCMeta):
    tag: str
    tag_qname: QName
    def drawrect(self, box: Box, is_active: bool) -> None: ...  # type: ignore[override]
    @abc.abstractmethod
    def el(self, box: Box) -> Element: ...

class SvgSquareDrawer(SvgQRModuleDrawer):
    unit_size: str
    def el(self, box: Box) -> Element: ...

class SvgCircleDrawer(SvgQRModuleDrawer):
    tag: str
    radius: str
    def el(self, box: Box) -> Element: ...

class SvgPathQRModuleDrawer(BaseSvgQRModuleDrawer, metaclass=abc.ABCMeta):
    img: SvgPathImage
    def drawrect(self, box: Box, is_active: bool) -> None: ...  # type: ignore[override]
    @abc.abstractmethod
    def subpath(self, box: Box) -> str: ...

class SvgPathSquareDrawer(SvgPathQRModuleDrawer):
    def subpath(self, box: Box) -> str: ...

class SvgPathCircleDrawer(SvgPathQRModuleDrawer):
    def subpath(self, box: Box) -> str: ...



from typing import Literal

def fp_str(*a): ...
def unicode2T1(utext, fonts): ...
def instanceStringWidthT1(self, text: str, size: float, encoding: str = "utf8") -> float: ...
def instanceStringWidthTTF(self, text: str, size: float, encoding: str = "utf8") -> float: ...
def hex32(i) -> str: ...
def add32(x: int, y: int) -> int: ...
def calcChecksum(data: str | bytes) -> int: ...
def escapePDF(s): ...
def asciiBase85Encode(input: str) -> str: ...
def asciiBase85Decode(input): ...
def sameFrag(f, g) -> bool | Literal[0]: ...

__all__ = [
    "fp_str",
    "unicode2T1",
    "instanceStringWidthT1",
    "instanceStringWidthTTF",
    "asciiBase85Encode",
    "asciiBase85Decode",
    "escapePDF",
    "sameFrag",
    "calcChecksum",
    "add32",
    "hex32",
]



from Xlib import (
    XK as XK,
    X as X,
    Xatom as Xatom,
    Xcursorfont as Xcursorfont,
    Xutil as Xutil,
    display as display,
    error as error,
    rdb as rdb,
)

__all__ = ["X", "XK", "Xatom", "Xcursorfont", "Xutil", "display", "error", "rdb"]

# Shared types throughout the stub



from _typeshed import SupportsRead
from pathlib import Path
from typing import Any, Literal

from PIL import Image

from .._types import Ink, Writeable
from ..main import ModulesType
from . import base
from .styles.colormasks import QRColorMask
from .styles.moduledrawers import SquareModuleDrawer
from .styles.moduledrawers.base import QRModuleDrawer

class StyledPilImage(base.BaseImageWithDrawer):
    kind: Literal["PNG"]
    color_mask: QRColorMask
    default_drawer_class: type[SquareModuleDrawer]
    embeded_image: Image.Image
    embeded_image_resample: Image.Resampling
    paint_color: Ink
    # the class accepts arbitrary additional positional arguments to accommodate
    # subclasses with additional arguments. kwargs are forwarded to the `new_image()` call
    # via the BaseImage.__init__ method.
    def __init__(
        self,
        border: int,
        width: int,
        box_size: int,
        *args: Any,
        qrcode_modules: ModulesType | None,
        module_drawer: QRModuleDrawer | str | None = None,
        eye_drawer: QRModuleDrawer | str | None = None,
        color_mask: QRColorMask = ...,
        embeded_image_path: str | bytes | Path | SupportsRead[bytes] | None = None,
        embeded_image: Image.Image | None = None,
        embeded_image_resample: Image.Resampling = ...,
        **kwargs: Any,
    ) -> None: ...
    # the new_image method accepts arbitrary keyword arguments to accommodate
    # subclasses with additional arguments.
    def new_image(self, **kwargs: Any) -> Image.Image: ...
    def draw_embedded_image(self) -> None: ...
    # kwargs are passed on to PIL.Image.save, which also accepts arbitrary keyword arguments.
    def save(  # type: ignore[override]
        self,
        stream: str | bytes | Path | Writeable,
        format: str | None = None,
        *,
        kind: str | None = None,
        save_all: bool = ...,
        bitmap_format: Literal["bmp", "png"] = ...,
        optimize: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    # attribute access is forwarded to the wrapped PIL.Image.Image instance.
    def __getattr__(self, name: str) -> Any: ...



from typing import ClassVar, NoReturn
from typing_extensions import deprecated

from .. import Command

@deprecated(
    """\
The test command is disabled and references to it are deprecated. \
Please remove any references to `setuptools.command.test` in all supported versions of the affected package.\
"""
)
class test(Command):
    description: ClassVar[str]
    user_options: ClassVar[list[tuple[str, str, str]]]
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> NoReturn: ...



from _typeshed import Incomplete
from typing import Final

from braintree.error_result import ErrorResult
from braintree.resource import Resource
from braintree.successful_result import SuccessfulResult

class DocumentUpload(Resource):
    class Kind:
        EvidenceDocument: Final = "evidence_document"

    @staticmethod
    def create(params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult: ...
    @staticmethod
    def create_signature() -> list[str]: ...
    def __init__(self, gateway, attributes) -> None: ...



import types
from collections.abc import Callable
from typing import Any, Final

import click

PY2: Final = False

def get_method_type(func: Callable[..., Any], obj: object) -> types.MethodType: ...
def get_choices(cli: click.Command, prog_name: str, args: list[str], incomplete: str) -> list[str]: ...



from braintree.graphql.enums import RecommendedPaymentOption

class PaymentRecommendation:
    payment_option: RecommendedPaymentOption
    recommended_priority: int
    def __init__(self, payment_option: RecommendedPaymentOption, recommended_priority: int) -> None: ...



from __future__ import annotations

from typing import Any, Hashable, Sequence, Union
from typing_extensions import assert_type


def check_update_method__int_key() -> None:
    d: dict[int, int] = {}
    d.update({1: 2})
    d.update([(1, 2)])
    d.update(a=3)  # type: ignore
    d.update({1: 2}, a=3)  # type: ignore
    d.update([(1, 2)], a=3)  # type: ignore
    d.update({"": 3})  # type: ignore
    d.update({1: ""})  # type: ignore
    d.update([("", 3)])  # type: ignore
    d.update([(3, "")])  # type: ignore


def check_update_method__str_key() -> None:
    d: dict[str, int] = {}
    d.update({"": 2})
    d.update([("", 2)])
    d.update(a=3)
    d.update({"": 2}, a=3)
    d.update([("", 2)], a=3)
    d.update({1: 3})  # type: ignore
    d.update({"": ""})  # type: ignore
    d.update([(1, 3)])  # type: ignore
    d.update([("", "")])  # type: ignore


def test_keywords_allowed_on_dict_update_where_key_type_is_str_supertype(
    a: dict[object, Any], b: dict[Hashable, Any], c: dict[Sequence[str], Any], d: dict[str, Any]
) -> None:
    a.update(keyword_args_are_accepted="whatever")
    b.update(here_too="whooo")
    c.update(and_here="hooray")
    d.update(also_here="yay")


def check_setdefault_method() -> None:
    d: dict[int, str] = {}
    d2: dict[int, str | None] = {}
    d3: dict[int, Any] = {}

    d.setdefault(1)  # type: ignore
    assert_type(d.setdefault(1, "x"), str)
    assert_type(d2.setdefault(1), Union[str, None])
    assert_type(d2.setdefault(1, None), Union[str, None])
    assert_type(d2.setdefault(1, "x"), Union[str, None])
    assert_type(d3.setdefault(1), Union[Any, None])
    assert_type(d3.setdefault(1, "x"), Any)



from typing import Final

class UnsupportedFormatException(Exception): ...
class InvalidCarrierException(Exception): ...
class SpanContextCorruptedException(Exception): ...

class Format:
    BINARY: Final = "binary"
    TEXT_MAP: Final = "text_map"
    HTTP_HEADERS: Final = "http_headers"



from _typeshed import Incomplete
from abc import ABCMeta
from builtins import bool as _bool
from typing import Any

import numpy as np
from tensorflow._aliases import DTypeLike
from tensorflow.python.framework.dtypes import HandleData

class _DTypeMeta(ABCMeta): ...

class DType(metaclass=_DTypeMeta):
    def __init__(self, type_enum: int, handle_data: HandleData | None = None) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def as_numpy_dtype(self) -> type[np.number[Any]]: ...
    @property
    def is_numpy_compatible(self) -> _bool: ...
    @property
    def is_bool(self) -> _bool: ...
    @property
    def is_floating(self) -> _bool: ...
    @property
    def is_integer(self) -> _bool: ...
    @property
    def is_quantized(self) -> _bool: ...
    @property
    def is_unsigned(self) -> _bool: ...
    def __getattr__(self, name: str) -> Incomplete: ...

bool: DType
complex128: DType
complex64: DType
bfloat16: DType
float16: DType
half: DType
float32: DType
float64: DType
double: DType
int8: DType
int16: DType
int32: DType
int64: DType
uint8: DType
uint16: DType
uint32: DType
uint64: DType
qint8: DType
qint16: DType
qint32: DType
quint8: DType
quint16: DType
string: DType

def as_dtype(type_value: DTypeLike) -> DType: ...
def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete
from collections.abc import Callable, Generator
from dataclasses import dataclass
from enum import Enum
from typing import Final, Literal
from typing_extensions import Self

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = [
    "minimum_spanning_edges",
    "maximum_spanning_edges",
    "minimum_spanning_tree",
    "maximum_spanning_tree",
    "number_of_spanning_trees",
    "random_spanning_tree",
    "partition_spanning_tree",
    "EdgePartition",
    "SpanningTreeIterator",
]

class EdgePartition(Enum):
    OPEN = 0
    INCLUDED = 1
    EXCLUDED = 2

@_dispatchable
def boruvka_mst_edges(G: Graph[_Node], minimum=True, weight="weight", keys=False, data=True, ignore_nan=False): ...
@_dispatchable
def kruskal_mst_edges(G: Graph[_Node], minimum, weight="weight", keys=True, data=True, ignore_nan=False, partition=None): ...
@_dispatchable
def prim_mst_edges(G: Graph[_Node], minimum, weight="weight", keys=True, data=True, ignore_nan=False): ...

ALGORITHMS: Final[dict[str, Callable[..., Generator[Incomplete, Incomplete, Incomplete]]]]

@_dispatchable
def minimum_spanning_edges(
    G: Graph[_Node],
    algorithm: str = "kruskal",
    weight: str = "weight",
    keys: bool = True,
    data: bool | None = True,
    ignore_nan: bool = False,
): ...
@_dispatchable
def maximum_spanning_edges(
    G: Graph[_Node],
    algorithm: str = "kruskal",
    weight: str = "weight",
    keys: bool = True,
    data: bool | None = True,
    ignore_nan: bool = False,
): ...
@_dispatchable
def minimum_spanning_tree(G: Graph[_Node], weight: str = "weight", algorithm: str = "kruskal", ignore_nan: bool = False): ...
@_dispatchable
def partition_spanning_tree(
    G: Graph[_Node], minimum: bool = True, weight: str = "weight", partition: str = "partition", ignore_nan: bool = False
): ...
@_dispatchable
def maximum_spanning_tree(G: Graph[_Node], weight: str = "weight", algorithm: str = "kruskal", ignore_nan: bool = False): ...
@_dispatchable
def random_spanning_tree(
    G: Graph[_Node], weight: str | None = None, *, multiplicative=True, seed: int | RandomState | None = None
): ...

class SpanningTreeIterator:
    @dataclass(order=True)
    class Partition:
        mst_weight: float
        partition_dict: dict[Incomplete, Incomplete]
        def __copy__(self) -> SpanningTreeIterator.Partition: ...

    G: Incomplete
    weight: Incomplete
    minimum: Incomplete
    ignore_nan: Incomplete
    partition_key: str

    def __init__(self, G: DiGraph[_Node], weight: str = "weight", minimum: bool = True, ignore_nan: bool = False) -> None: ...
    partition_queue: Incomplete

    def __iter__(self) -> Self: ...
    def __next__(self): ...

@_dispatchable
def number_of_spanning_trees(G: Graph[_Node], *, root=None, weight=None) -> float | Literal[0]: ...



import urllib.request
from typing import Any, Protocol, type_check_only

@type_check_only
class _HTTPClientProtocol(Protocol):  # noqa: Y046
    def download(
        self, uri: str, timeout: float | None = None, headers: dict[str, Any] = {}, verify_ssl: bool = True
    ) -> tuple[str, str]: ...

class DefaultHTTPClient:
    proxies: dict[str, str] | None

    def __init__(self, proxies: dict[str, str] | None = None) -> None: ...
    def download(
        self, uri: str, timeout: float | None = None, headers: dict[str, Any] = {}, verify_ssl: bool = True
    ) -> tuple[str, str]: ...

class HTTPSHandler:
    def __new__(cls, verify_ssl: bool = True) -> urllib.request.HTTPSHandler: ...  # type: ignore[misc]



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Token(VaultApiBase):
    def create(
        self,
        id=None,
        role_name=None,
        policies=None,
        meta=None,
        no_parent: bool = False,
        no_default_policy: bool = False,
        renewable: bool = True,
        ttl=None,
        type=None,
        explicit_max_ttl=None,
        display_name: str = "token",
        num_uses: int = 0,
        period=None,
        entity_alias=None,
        wrap_ttl=None,
        mount_point="token",
    ): ...
    def create_orphan(
        self,
        id=None,
        role_name=None,
        policies=None,
        meta=None,
        no_default_policy: bool = False,
        renewable: bool = True,
        ttl=None,
        type=None,
        explicit_max_ttl=None,
        display_name: str = "token",
        num_uses: int = 0,
        period=None,
        entity_alias=None,
        wrap_ttl=None,
        mount_point="token",
    ): ...
    def list_accessors(self, mount_point="token"): ...
    def lookup(self, token, mount_point="token"): ...
    def lookup_self(self, mount_point="token"): ...
    def lookup_accessor(self, accessor, mount_point="token"): ...
    def renew(self, token, increment=None, wrap_ttl=None, mount_point="token"): ...
    def renew_self(self, increment=None, wrap_ttl=None, mount_point="token"): ...
    def renew_accessor(self, accessor, increment=None, wrap_ttl=None, mount_point="token"): ...
    def revoke(self, token, mount_point="token"): ...
    def revoke_self(self, mount_point="token"): ...
    def revoke_accessor(self, accessor, mount_point="token"): ...
    def revoke_and_orphan_children(self, token, mount_point="token"): ...
    def read_role(self, role_name, mount_point="token"): ...
    def list_roles(self, mount_point="token"): ...
    def create_or_update_role(
        self,
        role_name,
        allowed_policies=None,
        disallowed_policies=None,
        orphan: bool = False,
        renewable: bool = True,
        path_suffix=None,
        allowed_entity_aliases=None,
        mount_point="token",
        token_period=None,
        token_explicit_max_ttl=None,
    ): ...
    def delete_role(self, role_name, mount_point="token"): ...
    def tidy(self, mount_point="token"): ...



from .authenticate_client import ClientAuthentication as ClientAuthentication
from .authorization_server import AuthorizationServer as AuthorizationServer
from .errors import (
    AccessDeniedError as AccessDeniedError,
    InsecureTransportError as InsecureTransportError,
    InvalidClientError as InvalidClientError,
    InvalidGrantError as InvalidGrantError,
    InvalidRequestError as InvalidRequestError,
    InvalidScopeError as InvalidScopeError,
    MismatchingStateException as MismatchingStateException,
    MissingAuthorizationError as MissingAuthorizationError,
    MissingCodeException as MissingCodeException,
    MissingTokenException as MissingTokenException,
    MissingTokenTypeException as MissingTokenTypeException,
    OAuth2Error as OAuth2Error,
    UnauthorizedClientError as UnauthorizedClientError,
    UnsupportedGrantTypeError as UnsupportedGrantTypeError,
    UnsupportedResponseTypeError as UnsupportedResponseTypeError,
    UnsupportedTokenTypeError as UnsupportedTokenTypeError,
)
from .grants import (
    AuthorizationCodeGrant as AuthorizationCodeGrant,
    AuthorizationEndpointMixin as AuthorizationEndpointMixin,
    BaseGrant as BaseGrant,
    ClientCredentialsGrant as ClientCredentialsGrant,
    ImplicitGrant as ImplicitGrant,
    RefreshTokenGrant as RefreshTokenGrant,
    ResourceOwnerPasswordCredentialsGrant as ResourceOwnerPasswordCredentialsGrant,
    TokenEndpointMixin as TokenEndpointMixin,
)
from .models import AuthorizationCodeMixin as AuthorizationCodeMixin, ClientMixin as ClientMixin, TokenMixin as TokenMixin
from .requests import (
    JsonPayload as JsonPayload,
    JsonRequest as JsonRequest,
    OAuth2Payload as OAuth2Payload,
    OAuth2Request as OAuth2Request,
)
from .resource_protector import ResourceProtector as ResourceProtector, TokenValidator as TokenValidator
from .token_endpoint import TokenEndpoint as TokenEndpoint
from .util import list_to_scope as list_to_scope, scope_to_list as scope_to_list
from .wrappers import OAuth2Token as OAuth2Token

__all__ = [
    "OAuth2Payload",
    "OAuth2Token",
    "OAuth2Request",
    "JsonPayload",
    "JsonRequest",
    "OAuth2Error",
    "AccessDeniedError",
    "MissingAuthorizationError",
    "InvalidGrantError",
    "InvalidClientError",
    "InvalidRequestError",
    "InvalidScopeError",
    "InsecureTransportError",
    "UnauthorizedClientError",
    "UnsupportedResponseTypeError",
    "UnsupportedGrantTypeError",
    "UnsupportedTokenTypeError",
    "MissingCodeException",
    "MissingTokenException",
    "MissingTokenTypeException",
    "MismatchingStateException",
    "ClientMixin",
    "AuthorizationCodeMixin",
    "TokenMixin",
    "ClientAuthentication",
    "AuthorizationServer",
    "ResourceProtector",
    "TokenValidator",
    "TokenEndpoint",
    "BaseGrant",
    "AuthorizationEndpointMixin",
    "TokenEndpointMixin",
    "AuthorizationCodeGrant",
    "ImplicitGrant",
    "ResourceOwnerPasswordCredentialsGrant",
    "ClientCredentialsGrant",
    "RefreshTokenGrant",
    "scope_to_list",
    "list_to_scope",
]



from _typeshed import Incomplete
from collections.abc import Callable
from typing import Literal

from .tokens import OAuth2Token

def prepare_grant_uri(
    uri: str,
    client_id: str,
    response_type: Literal["code", "token"],
    redirect_uri: str | None = None,
    scope: str | set[object] | tuple[object] | list[object] | None = None,
    state: str | None = None,
    code_challenge: str | None = None,
    code_challenge_method: str | None = "plain",
    **kwargs,
) -> str: ...
def prepare_token_request(
    grant_type: str,
    body: str = "",
    include_client_id: bool = True,
    code_verifier: str | None = None,
    *,
    scope: str | set[object] | tuple[object] | list[object] | None = None,
    client_id: str | None = None,
    client_secret: str | None = None,
    code: str | None = None,
    redirect_uri: str | None = None,
    **kwargs,
) -> str: ...
def prepare_token_revocation_request(
    url: str,
    token: str,
    token_type_hint: Literal["access_token", "refresh_token"] | None = "access_token",
    callback: Callable[[Incomplete], Incomplete] | None = None,
    body: str = "",
    **kwargs,
) -> tuple[str, dict[str, str], str]: ...
def parse_authorization_code_response(uri: str, state: str | None = None) -> dict[str, str]: ...
def parse_implicit_response(
    uri: str, state: str | None = None, scope: str | set[object] | tuple[object] | list[object] | None = None
) -> OAuth2Token: ...
def parse_token_response(
    body: str | bytes | bytearray, scope: str | set[object] | tuple[object] | list[object] | None = None
) -> OAuth2Token: ...
def validate_token_parameters(params: dict[str, Incomplete]) -> None: ...
def parse_expires(params: dict[str, Incomplete]) -> tuple[int | None, float | None, float | None]: ...



from _typeshed import BytesPath, StrOrBytesPath, StrPath
from collections.abc import Callable, Generator, Iterable
from os import PathLike
from types import TracebackType
from typing import IO, Any, NoReturn, TypeVar, overload
from typing_extensions import Self

_StrPathT = TypeVar("_StrPathT", bound=StrPath)
_BytesPathT = TypeVar("_BytesPathT", bound=BytesPath)

def mkdir_p(path: StrOrBytesPath) -> None: ...
def rotate_file(filename: PathLike[str], *, keep: int = 5) -> None: ...

class FilePerms:
    user: str
    group: str
    other: str
    def __init__(self, user: str = "", group: str = "", other: str = "") -> None: ...
    @classmethod
    def from_int(cls, i: int) -> Self: ...
    @classmethod
    def from_path(cls, path: StrOrBytesPath) -> Self: ...
    def __int__(self) -> int: ...

def atomic_save(dest_path: str, **kwargs) -> AtomicSaver: ...

class AtomicSaver:
    dest_path: str
    overwrite: bool
    file_perms: int | None
    overwrite_part: bool
    part_filename: str | None
    rm_part_on_exc: bool
    text_mode: bool
    buffering: int
    dest_dir: str
    part_path: str
    mode: str
    open_flags: int
    part_file: str | None
    def __init__(self, dest_path: str, **kwargs) -> None: ...
    def setup(self) -> None: ...
    def __enter__(self) -> IO[Any] | None: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

def iter_find_files(
    directory: str,
    patterns: str | Iterable[str],
    ignored: str | Iterable[str] | None = None,
    include_dirs: bool = False,
    max_depth: int | None = None,
) -> Generator[str, None, None]: ...
@overload
def copy_tree(
    src: _StrPathT, dst: _StrPathT, symlinks: bool = False, ignore: Callable[[_StrPathT, list[str]], Iterable[str]] | None = None
) -> None: ...
@overload
def copy_tree(
    src: _BytesPathT,
    dst: _BytesPathT,
    symlinks: bool = False,
    ignore: Callable[[_BytesPathT, list[bytes]], Iterable[bytes]] | None = None,
) -> None: ...

copytree = copy_tree

class DummyFile:
    name: StrOrBytesPath
    mode: str
    closed: bool
    errors: None
    isatty: bool
    encoding: None
    newlines: None
    softspace: int
    def __init__(self, path: StrOrBytesPath, mode: str = "r", buffering: int | None = None) -> None: ...
    def close(self) -> None: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def next(self) -> NoReturn: ...
    def read(self, size: int = 0) -> str: ...
    def readline(self, size: int = 0) -> str: ...
    def readlines(self, size: int = 0) -> list[str]: ...
    def seek(self) -> None: ...
    def tell(self) -> int: ...
    def truncate(self) -> None: ...
    def write(self, string: str) -> None: ...
    def writelines(self, list_of_strings: list[str]) -> None: ...
    def __next__(self) -> NoReturn: ...
    def __enter__(self) -> None: ...
    def __exit__(self, exc_type, exc_val, exc_tb) -> None: ...

__all__ = ["mkdir_p", "atomic_save", "AtomicSaver", "FilePerms", "iter_find_files", "copytree"]



from typing import TypedDict, type_check_only
from typing_extensions import NotRequired

from .sampling_rule import SamplingRule, _Rule

@type_check_only
class _SamplingRule(TypedDict):
    version: NotRequired[int]
    default: _Rule
    rules: list[_Rule]

local_sampling_rule: _SamplingRule
SUPPORTED_RULE_VERSION: tuple[int, ...]

class LocalSampler:
    def __init__(self, rules: _SamplingRule = ...) -> None: ...
    def should_trace(self, sampling_req: SamplingRule | None = None) -> bool: ...
    def load_local_rules(self, rules: _SamplingRule) -> None: ...



from collections.abc import Callable
from typing import Literal, overload

def require_string(value: object, value_name: str) -> None: ...  # raise TypeError if value is not string
def require_callback(
    callback: object, callback_name: str = "callback"
) -> None: ...  # raise TypeError if callback is not callable
@overload
def rpc_completion_callback(callback: None) -> Literal[True]: ...
@overload
def rpc_completion_callback(callback: Callable[..., object]) -> Literal[False]: ...
def zero_or_greater(name: str, value: str | float) -> None: ...



class RenderPMError(Exception): ...

def setFont(gs, fontName, fontSize) -> None: ...
def pathNumTrunc(n): ...
def text2Path(
    text,
    x: int = 0,
    y: int = 0,
    fontName="Times-Roman",
    fontSize: int = 1000,
    anchor: str = "start",
    truncate: int = 1,
    pathReverse: int = 0,
    gs=None,
    **kwds,
): ...

# NOTE: This only exists on some render backends
def processGlyph(G, truncate=1, pathReverse=0): ...
def text2PathDescription(text, x=0, y=0, fontName=..., fontSize=1000, anchor="start", truncate=1, pathReverse=0, gs=None): ...

__all__ = ("setFont", "pathNumTrunc", "processGlyph", "text2PathDescription", "text2Path", "RenderPMError")



from _typeshed import Incomplete
from typing import IO, Final

from reportlab.graphics.renderbase import Renderer
from reportlab.graphics.shapes import Drawing
from reportlab.pdfgen.canvas import Canvas

__version__: Final[str]
PS_WinAnsiEncoding: Final[str]

class PSCanvas:
    comments: int
    code: Incomplete
    code_append: Incomplete
    PostScriptLevel: Incomplete
    def __init__(self, size=(300, 300), PostScriptLevel: int = 2) -> None: ...
    def comment(self, msg) -> None: ...
    def drawImage(self, image, x1, y1, width=None, height=None) -> None: ...
    def clear(self) -> None: ...
    def save(self, f=None) -> None: ...
    def saveState(self) -> None: ...
    def restoreState(self) -> None: ...
    def stringWidth(self, s, font=None, fontSize=None): ...
    def setLineCap(self, v) -> None: ...
    def setLineJoin(self, v) -> None: ...
    def setDash(self, array=[], phase: int = 0) -> None: ...
    def setStrokeColor(self, color) -> None: ...
    def setColor(self, color) -> None: ...
    def setFillColor(self, color) -> None: ...
    def setFillMode(self, v) -> None: ...
    def setLineWidth(self, width) -> None: ...
    def setFont(self, font, fontSize, leading=None) -> None: ...
    def line(self, x1, y1, x2, y2) -> None: ...
    def drawString(self, x, y, s, angle: int = 0, text_anchor: str = "left", textRenderMode: int = 0) -> None: ...
    def drawCentredString(self, x, y, text, text_anchor: str = "middle", textRenderMode: int = 0) -> None: ...
    def drawRightString(self, text, x, y, text_anchor: str = "end", textRenderMode: int = 0) -> None: ...
    def drawCurve(self, x1, y1, x2, y2, x3, y3, x4, y4, closed: int = 0) -> None: ...
    def rect(self, x1, y1, x2, y2, stroke: int = 1, fill: int = 1) -> None: ...
    def roundRect(self, x1, y1, x2, y2, rx: int = 8, ry: int = 8) -> None: ...
    def ellipse(self, x1, y1, x2, y2) -> None: ...
    def circle(self, xc, yc, r) -> None: ...
    def drawArc(self, x1, y1, x2, y2, startAng: int = 0, extent: int = 360, fromcenter: int = 0) -> None: ...
    def polygon(self, p, closed: int = 0, stroke: int = 1, fill: int = 1) -> None: ...
    def lines(self, lineList, color=None, width=None) -> None: ...
    def moveTo(self, x, y) -> None: ...
    def lineTo(self, x, y) -> None: ...
    def curveTo(self, x1, y1, x2, y2, x3, y3) -> None: ...
    def closePath(self) -> None: ...
    def polyLine(self, p) -> None: ...
    def drawFigure(self, partList, closed: int = 0) -> None: ...
    def translate(self, x, y) -> None: ...
    def scale(self, x, y) -> None: ...
    def transform(self, a, b, c, d, e, f) -> None: ...

def draw(drawing: Drawing, canvas: Canvas, x: float = 0, y: float = 0, showBoundary=0) -> None: ...

class _PSRenderer(Renderer):
    def drawNode(self, node) -> None: ...
    def drawRect(self, rect) -> None: ...
    def drawLine(self, line) -> None: ...
    def drawCircle(self, circle) -> None: ...
    def drawWedge(self, wedge) -> None: ...
    def drawPolyLine(self, p) -> None: ...
    def drawEllipse(self, ellipse) -> None: ...
    def drawPolygon(self, p) -> None: ...
    def drawString(self, stringObj) -> None: ...
    def drawPath(self, path, fillMode=None): ...
    def applyStateChanges(self, delta, newState) -> None: ...
    def drawImage(self, image) -> None: ...

def drawToFile(d: Drawing, fn: IO[bytes], showBoundary=0, **kwd) -> None: ...
def drawToString(d: Drawing, showBoundary=0) -> str: ...
def test(outDir: str = "epsout", shout: bool = False) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node

__all__ = ["cuthill_mckee_ordering", "reverse_cuthill_mckee_ordering"]

def cuthill_mckee_ordering(G: Graph[_Node], heuristic=None) -> Generator[Incomplete, Incomplete, None]: ...
def reverse_cuthill_mckee_ordering(G: Graph[_Node], heuristic=None): ...
def connected_cuthill_mckee_ordering(G: Graph[_Node], heuristic=None): ...
def pseudo_peripheral_node(G: Graph[_Node]): ...
